# Copyright (C) 2021-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class admv8818(attribute, context_manager):
    """ADMV8818 2 GHz to 18 GHz, Digitally Tunable, High-Pass and Low-Pass Filter

    parameters:
        uri: type=string
            URI of IIO context with ADMV8818
    """

    _device_name = "admv8818"

    def __init__(self, uri="", device_name=""):

        context_manager.__init__(self, uri, self._device_name)

        self._device_name = device_name
        # Find the device
        self._ctrl = self._ctx.find_device(self._device_name)

        # Raise an exception if the device isn't found
        if not self._ctrl:
            raise Exception("ADMV8818 device not found")

    @property
    def low_pass_3db_frequency(self):
        """Get/Set the Low Pass 3dB Frequency in MHz"""
        return self._get_iio_attr(
            "altvoltage0", "filter_low_pass_3db_frequency", True, self._ctrl
        )

    @low_pass_3db_frequency.setter
    def low_pass_3db_frequency(self, value):
        """Get/Set the Low Pass 3dB Frequency in MHz"""
        self._set_iio_attr_int(
            "altvoltage0", "filter_low_pass_3db_frequency", True, int(value), self._ctrl
        )

    @property
    def high_pass_3db_frequency(self):
        """Get/Set the Low Pass 3dB Frequency in MHz"""
        return self._get_iio_attr(
            "altvoltage0", "filter_high_pass_3db_frequency", True, self._ctrl
        )

    @high_pass_3db_frequency.setter
    def high_pass_3db_frequency(self, value):
        """Get/Set the Low Pass 3dB Frequency in MHz"""
        self._set_iio_attr_int(
            "altvoltage0",
            "filter_high_pass_3db_frequency",
            True,
            int(value),
            self._ctrl,
        )

    @property
    def band_pass_bandwidth_3db_frequency(self):
        """Get/Set the Band Pass 3dB Frequency in MHz"""
        return self._get_iio_attr(
            "altvoltage0", "filter_band_pass_bandwidth_3db_frequency", True, self._ctrl
        )

    @band_pass_bandwidth_3db_frequency.setter
    def band_pass_bandwidth_3db_frequency(self, value):
        """Get/Set the Band Pass 3dB Frequency in MHz"""
        self._set_iio_attr_int(
            "altvoltage0",
            "filter_band_pass_bandwidth_3db_frequency",
            True,
            int(value),
            self._ctrl,
        )

    @property
    def band_pass_center_frequency(self):
        """Get/Set the Band Pass Center Frequency in MHz"""
        return self._get_iio_attr(
            "altvoltage0", "filter_band_pass_center_frequency", True, self._ctrl
        )

    @band_pass_center_frequency.setter
    def band_pass_center_frequency(self, value):
        """Get/Set the Band Pass Center Frequency in MHz"""
        self._set_iio_attr_int(
            "altvoltage0",
            "filter_band_pass_center_frequency",
            True,
            int(value),
            self._ctrl,
        )

    @property
    def mode_available(self):
        """Get available modes"""
        return self._get_iio_attr_str("altvoltage0", "mode_available", True, self._ctrl)

    @property
    def mode(self):
        """Get/Set mode"""
        return self._get_iio_attr_str("altvoltage0", "mode", True, self._ctrl)

    @mode.setter
    def mode(self, value):
        """Get/Set mode"""
        self._set_iio_attr("altvoltage0", "mode", True, value, self._ctrl)

    def reg_read(self, reg):
        """Direct Register Access via debugfs"""
        self._set_iio_debug_attr_str("direct_reg_access", reg, self._ctrl)
        return self._get_iio_debug_attr_str("direct_reg_access", self._ctrl)

    def reg_write(self, reg, value):
        """Direct Register Access via debugfs"""
        self._set_iio_debug_attr_str("direct_reg_access", f"{reg} {value}", self._ctrl)



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.context_manager import context_manager
from adi.jesd import jesd
from adi.rx_tx import tx
from adi.sync_start import sync_start


class ad9162(tx, context_manager, sync_start):
    """ AD9162 16-Bit, 12 GSPS, RF DAC """

    _complex_data = False
    # _complex_data = True
    _tx_channel_names = ["voltage0_i", "voltage0_q"]
    _device_name = ""

    def __init__(self, uri="", username="root", password="analog"):

        context_manager.__init__(self, uri, self._device_name)

        self._jesd = jesd(uri, username=username, password=password)
        self._txdac = self._ctx.find_device("axi-ad9162-hpc")

        tx.__init__(self)

    @property
    def fir85_enable(self):
        return self._get_iio_attr("voltage0_i", "fir85_enable", True, self._txdac)

    @fir85_enable.setter
    def fir85_enable(self, value):
        self._set_iio_attr("voltage0_i", "fir85_enable", True, value, self._txdac)

    @property
    def sample_rate(self):
        """sample_rate: Sample frequency rate TX path in samples per second."""
        return self._get_iio_attr("voltage0_i", "sampling_frequency", True, self._txdac)

    @property
    def scale(self):
        return self._get_iio_attr("voltage0_i", "scale", True, self._txdac)

    @property
    def frequency_nco(self):
        return self._get_iio_attr("altvoltage4", "frequency_nco", True, self._txdac)

    @frequency_nco.setter
    def frequency_nco(self, value):
        self._set_iio_attr("altvoltage4", "frequency_nco", True, value, self._txdac)

    @property
    def jesd204_statuses(self):
        return self._jesd.get_all_statuses()



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class adl5240(attribute, context_manager):
    """ ADL5240 100 MHz TO 4000 MHz RF/IF Digitally Controlled VGA """

    _device_name = "adl5240"

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)

        self._ctrl = self._ctx.find_device(self._device_name)

        if not self._ctrl:
            raise Exception("ADL5240 device not found")

    @property
    def hardwaregain(self):
        """hardwaregain: Attenuation applied to TX path"""
        return self._get_iio_attr("voltage0", "hardwaregain", True)

    @hardwaregain.setter
    def hardwaregain(self, value):
        self._set_iio_attr_float("voltage0", "hardwaregain", True, value, self._ctrl)



# Copyright (C) 2020-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class _dyn_property:
    """ Class descriptor for accessing individual attributes """

    def __init__(self, attr, dev, channel_name, output):
        self._dev = dev
        self._attr = attr
        self._channel_name = channel_name
        self._output = output

    def __get__(self, instance, owner):
        return instance._get_iio_attr(
            self._channel_name, self._attr, self._output, self._dev
        )

    def __set__(self, instance, value):
        instance._set_iio_attr_int(
            self._channel_name, self._attr, self._output, int(value), self._dev
        )


class one_bit_adc_dac(attribute, context_manager):
    """One bit ADC/DAC (GPIO). This driver will create a handle for the
        GPIO device as well as properties for each GPIO pin it accesses.
        Each GPIO pin name will be lowercase and of the format:
        "gpio_{pin name}"

    parameters:
        uri: type=string
            URI of IIO context with GPIO pins
        name: type=string
            String identifying the device by name from the device tree.
            Dynamic class properties will be created for each channel.
    """

    _device_name = ""

    def __init__(self, uri="", name="one-bit-adc-dac"):

        try:
            context_manager.__init__(self, uri, self._device_name)
        except Exception:
            raise Exception(f"No device found for {name}")

        self._ctrl = None

        for dev in self._ctx.devices:
            if "label" in dev.attrs and dev.attrs["label"].value == name:
                self._ctrl = dev
                break
            else:
                if dev.name == name:
                    self._ctrl = dev
                    break

        if not self._ctrl:
            raise Exception(f"No device found for {name}")

        for chan in self._ctrl.channels:
            setattr(
                type(self),
                f"gpio_{chan.attrs['label'].value.lower()}",
                _dyn_property(
                    "raw", dev=self._ctrl, channel_name=chan.id, output=chan.output
                ),
            )



# Copyright (C) 2020-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

import numpy as np
from adi.context_manager import context_manager
from adi.rx_tx import rx


class ad7768(rx, context_manager):

    """ AD7768 8-channel, Simultaneous Sampling Sigma-Delta ADC """

    _device_name = " "
    _rx_data_type = np.int32

    def __init__(
        self, uri="ip:analog.local",
    ):
        """Initialize."""
        context_manager.__init__(self, uri, self._device_name)

        self._ctrl = self._ctx.find_device("ad7768")
        self._rxadc = self._ctx.find_device("ad7768")
        self._device_name = "ad7768"

        if not self._rxadc:
            self._ctrl = self._ctx.find_device("cf_axi_adc")
            self._rxadc = self._ctx.find_device("cf_axi_adc")
            self._device_name = "cf_axi_adc"

        if not self._ctrl:
            raise Exception("Error in selecting matching device")

        if not self._rxadc:
            raise Exception("Error in selecting matching device")

        self._rx_channel_names = []
        for ch in self._rxadc.channels:
            name = ch._id
            self._rx_channel_names.append(name)

        rx.__init__(self)

    @property
    def sampling_frequency_available(self):
        """Get available sampling frequencies."""
        return self._get_iio_dev_attr("sampling_frequency_available")

    @property
    def sampling_frequency(self):
        """Get sampling frequency."""
        return self._get_iio_dev_attr("sampling_frequency")

    @sampling_frequency.setter
    def sampling_frequency(self, rate):
        """Set sampling frequency."""
        if rate in self.sampling_frequency_available:
            self._set_iio_dev_attr("sampling_frequency", rate)
        else:
            raise ValueError(
                "Error: Sampling frequency not supported \nUse one of: "
                + str(self.sampling_frequency_available)
            )

    @property
    def power_mode_avail(self):
        """Get available power modes."""
        return self._get_iio_dev_attr_str("power_mode_available")

    @property
    def power_mode(self):
        """Get power mode."""
        return self._get_iio_dev_attr_str("power_mode")

    @power_mode.setter
    def power_mode(self, mode):
        """Set power mode."""
        if mode in self.power_mode_avail:
            self._set_iio_dev_attr_str("power_mode", mode)
        else:
            raise ValueError(
                "Error: Power mode not supported \nUse one of: "
                + str(self.power_mode_avail)
            )

    @property
    def filter_type_avail(self):
        """Get available filter types."""
        return self._get_iio_dev_attr_str("filter_type_available")

    @property
    def filter_type(self):
        """Get filter type."""
        return self._get_iio_dev_attr_str("filter_type")

    @filter_type.setter
    def filter_type(self, ftype):
        """Set filter type."""
        if ftype in self.filter_type_avail:
            self._set_iio_dev_attr_str("filter_type", ftype)
        else:
            raise ValueError(
                "Error: Filter type not supported \nUse one of: "
                + str(self.filter_type_avail)
            )


class ad7768_4(ad7768):

    """ AD7768 4-channel, Simultaneous Sampling Sigma-Delta ADC """

    @property
    def sync_start_enable_available(self):
        """Get available sync start enable types."""
        return self._get_iio_dev_attr_str("sync_start_enable_available")

    @property
    def sync_start_enable(self):
        """Get sync start enable."""
        return self._get_iio_dev_attr_str("sync_start_enable")

    @sync_start_enable.setter
    def sync_start_enable(self, ftype):
        """Set sync start enable."""
        if ftype in self.sync_start_enable_available:
            self._set_iio_dev_attr_str("sync_start_enable", ftype)
        else:
            raise ValueError(
                "Error: Sync start enable not supported \nUse one of: "
                + str(self.sync_start_enable_available)
            )



# Copyright (C) 2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD
import adi


class cn0556(adi.cn0554):

    """The CN0556 class inherits features from the CN0554 (providing full
    control and monitoring of the input and output voltages and currents)
    and the one_bit_adc_dac (sets the mode of the regulator to either Buck
    or Boost and enables and disables the LT8228). These combined
    functionalities are utilized for a Programmable High Current and
    Voltage Source/Sink Power Supply.

    parameters:
        uri: type=string
            URI of the platform
    """

    _board_config = {
        "buck_input_current_limit": {
            "scale": -1.2174 / 1000.0,
            "offset": 11.345,
            "min": 0.07,
            "max": 10.0,
            "unit": "A"
            # I1P = 11.345-1.1274*ISET1P_CTL
        },
        "buck_output_current_limit": {
            "scale": -4.0108 / 1000.0,
            "offset": 39.623,
            "min": 0.0,
            "max": 35.0,
            "unit": "A"
            # I2P = 39.623-4.0108*ISET2P_CTL
        },
        "buck_input_undervoltage": {
            "scale": -5.1281 / 1000.0,
            "offset": 60.415,
            "min": 12.0,
            "max": 54.0,
            "unit": "V"
            # VUV1 = 60.415-5.1281*UV1_CTL
        },
        "buck_target_output_voltage": {
            "scale": -1.4666 / 1000.0,
            "offset": 15.685,
            "min": 2.0,
            "max": 14.0,
            "unit": "V"
            # V2D = 15.685-1.4666*FB2_CTL
        },
        "boost_input_current_limit": {
            "scale": -4.0108 / 1000.0,
            "offset": 39.623,
            "min": 0.0,
            "max": 35.0,
            "unit": "A"
            # I2N = 39.623-4.0108*ISET2N_CTL
        },
        "boost_output_current_limit": {
            "scale": -1.1274 / 1000.0,
            "offset": 11.345,
            "min": 0.07,
            "max": 10.0,
            "unit": "A"
            # I1N = 11.345-1.1274*ISET1N_CTL
        },
        "boost_input_undervoltage": {
            "scale": -1.2195 / 1000.0,
            "offset": 13.386,
            "min": 8.0,
            "max": 12.0,
            "unit": "V"
            # VUV2 = 13.386-1.2195*UV2_CTL
        },
        "boost_target_output_voltage": {
            "scale": -4.859 / 1000.0,
            "offset": 61.995,
            "min": 14.0,
            "max": 56.0,
            "unit": "V"
            # V2D = 61.995-4.859*FB1_CTL
        },
        # Measurements
        "buck_input_voltage": {
            "scale": 243.0 / 1000.0,
            "offset": 0.0,
            # V1 = V1_ADC*(243k/10k)
        },
        "buck_input_current": {
            "scale": 4.0 / 100.0,
            "offset": 0.0,
            # IV1 = 4*VMON1
        },
        "buck_output_current": {
            "scale": 14.0 / 100.0,
            "offset": 0.0,
            # IV2 = 14*VMON1
        },
        "buck_output_voltage": {
            "scale": 51.0 / 1000.0,
            "offset": 0.0,
            # V2 = V2_ADC*(51k/10k)
        },
        "boost_input_voltage": {
            "scale": 51.0 / 1000.0,
            "offset": 0.0,
            # V2 = V2_ADC*(51k/10k)
        },
        "boost_input_current": {
            "scale": 14.0 / 100.0,
            "offset": 0.0,
            # IV2 = 14*VMON1
        },
        "boost_output_current": {
            "scale": 4.0 / 100.0,
            "offset": 0.0,
            # IV1 = 4*VMON1
        },
        "boost_output_voltage": {
            "scale": 243.0 / 1000.0,
            "offset": 0.0,
            # V1 = V1_ADC*(243k/10k)
        },
        "intvcc_voltage": {"scale": 1 / 100.0, "offset": 0.0},
        "share_voltage": {"scale": 1 / 100.0, "offset": 0.0},
    }

    def __init__(self, uri="ip:analog.local"):
        adi.cn0554.__init__(self, uri=uri)
        self.gpio = adi.one_bit_adc_dac(uri=uri)

        # Enable ADC Channels for reading Voltage and Current Monitoring, INTVCC, SHARE
        self.rx_enabled_channels = [0, 4, 8, 10, 12, 14]
        self.adc_scale = self.adc.channel[0].scale

    @property
    def drxn(self):
        """drxn: operating mode of the board. Returns '1' if in Buck Mode and '0' if in Boost Mode"""
        return self.gpio.gpio_drxn

    @drxn.setter
    def drxn(self, value):
        """drxn: Buck or Boost Mode Setter. Options are: 1 (Buck Mode), 0 (Boost Mode)."""
        if value != 1 and value != 0:
            raise Exception(
                "Invalid Input. Valid values are '1' for Buck Mode and '0' for Boost Mode"
            )

        self.gpio.gpio_drxn = value

    @property
    def enable(self):
        """enable the LT8228 device when True.
        When set to false, device is disabled, DAC outputs set to 0, and sets the DRXN to boost mode."""
        return self.gpio.gpio_en

    @enable.setter
    def enable(self, value):
        if value:
            self.gpio.gpio_en = 1
        else:
            self.dac.voltage0.volt = 0
            self.dac.voltage2.volt = 0
            self.dac.voltage4.volt = 0
            self.dac.voltage6.volt = 0
            self.dac.voltage8.volt = 0
            self.dac.voltage10.volt = 0
            self.dac.voltage12.volt = 0
            self.dac.voltage14.volt = 0
            self.gpio.gpio_en = 0
            self.drxn = 0

    @property
    def intvcc_voltage(self):
        """Return voltage at INTVCC pin in volts (V)"""
        return self.read_value(
            self.adc.channel[10],
            "raw",
            self._board_config["intvcc_voltage"]["scale"] * self.adc_scale,
            self._board_config["intvcc_voltage"]["offset"],
        )

    @property
    def share_voltage(self):
        """Return voltage at SHARE pin in volts (V)"""
        return self.read_value(
            self.adc.channel[12],
            "raw",
            self._board_config["share_voltage"]["scale"] * self.adc_scale,
            self._board_config["share_voltage"]["offset"],
        )

    @property
    def report(self):
        """Check if REPORT is triggered. Returns True if REPORT pin is HIGH."""
        return self.gpio.gpio_report

    @property
    def fault(self):
        """fault: checks if a fault is present. Returns True if a fault occurred."""
        return self.gpio.gpio_fault

    @property
    def buck_output_voltage(self):
        """Read buck output voltage at V2 terminals. Convert ADC data using scale and offset factors based on resistor divider network."""
        return self.read_value(
            self.adc.channel[4],
            "raw",
            self._board_config["buck_output_voltage"]["scale"] * self.adc_scale,
            self._board_config["buck_output_voltage"]["offset"],
        )

    @property
    def buck_target_output_voltage(self):
        """
        Compute for the target buck output voltage set at V2 side
        using DAC data and scale and offset factors based on resistor divider network at FB2 node.

        returns buck target output voltage and DAC output voltage in volts (V)
        return type: (float, float)

        """
        return (
            self.read_value(
                self.dac.voltage2,
                "volt",
                self._board_config["buck_target_output_voltage"]["scale"],
                self._board_config["buck_target_output_voltage"]["offset"],
            ),
            self.dac.voltage2.volt / 1000,
        )

    @buck_target_output_voltage.setter
    def buck_target_output_voltage(self, value):
        """Set buck output voltage at V2 side. Valid values are 2 to 14."""
        try:
            self.set_value(
                value,
                self.dac.voltage2,
                "volt",
                self._board_config["buck_target_output_voltage"]["scale"],
                self._board_config["buck_target_output_voltage"]["offset"],
                self._board_config["buck_target_output_voltage"]["min"],
                self._board_config["buck_target_output_voltage"]["max"],
                self._board_config["buck_target_output_voltage"]["unit"],
            )
        except Exception as e:
            raise e

    @property
    def buck_input_voltage(self):
        """Read buck input voltage at V1 terminals. Convert ADC data using scale and offset factors based on a resistor divider network."""
        return self.read_value(
            self.adc.channel[14],
            "raw",
            self._board_config["buck_input_voltage"]["scale"] * self.adc_scale,
            self._board_config["buck_input_voltage"]["offset"],
        )

    @property
    def buck_output_current(self):
        """Read buck output current at V2 terminals. Convert ADC data using scale and offset factors based on resistor divider at IMON2 node."""
        return self.read_value(
            self.adc.channel[8],
            "raw",
            self._board_config["buck_output_current"]["scale"] * self.adc_scale,
            self._board_config["buck_output_current"]["offset"],
        )

    @property
    def buck_input_current(self):
        """Read buck input current at V1 terminals. Convert ADC data using scale and offset factors based on resistor divider at IMON1 node."""
        return self.read_value(
            self.adc.channel[0],
            "raw",
            self._board_config["buck_input_current"]["scale"] * self.adc_scale,
            self._board_config["buck_input_current"]["offset"],
        )

    @property
    def buck_input_undervoltage(self):
        """
        Compute for the buck input undervoltage set at V1 side
        using DAC data and scale and offset factors based on resistor divider network at UV1 node.

        returns buck input undervoltage and DAC output voltage in volts (V)
        return type: (float, float)

        """

        return (
            self.read_value(
                self.dac.voltage4,
                "volt",
                self._board_config["buck_input_undervoltage"]["scale"],
                self._board_config["buck_input_undervoltage"]["offset"],
            ),
            self.dac.voltage4.volt / 1000,
        )

    @buck_input_undervoltage.setter
    def buck_input_undervoltage(self, value):
        """Set buck input undervoltage V1 side. Valid values are 12 to 54."""
        try:
            self.set_value(
                value,
                self.dac.voltage4,
                "volt",
                self._board_config["buck_input_undervoltage"]["scale"],
                self._board_config["buck_input_undervoltage"]["offset"],
                self._board_config["buck_input_undervoltage"]["min"],
                self._board_config["buck_input_undervoltage"]["max"],
                self._board_config["buck_input_undervoltage"]["unit"],
            )
        except Exception as e:
            raise e

    @property
    def buck_input_current_limit(self):
        """
        Compute for the target buck input current limit set
        using DAC data and scale and offset factors based on resistor divider network at ISET1P node.

        returns buck target input current limit and DAC output voltage in volts (V)
        return type: (float, float)

        """
        return (
            self.read_value(
                self.dac.voltage8,
                "volt",
                self._board_config["buck_input_current_limit"]["scale"],
                self._board_config["buck_input_current_limit"]["offset"],
            ),
            self.dac.voltage8.volt / 1000,
        )

    @buck_input_current_limit.setter
    def buck_input_current_limit(self, value):
        """Set buck input current limit V1 side. Valid values are 0.07 to 10."""
        try:
            self.set_value(
                value,
                self.dac.voltage8,
                "volt",
                self._board_config["buck_input_current_limit"]["scale"],
                self._board_config["buck_input_current_limit"]["offset"],
                self._board_config["buck_input_current_limit"]["min"],
                self._board_config["buck_input_current_limit"]["max"],
                self._board_config["buck_input_current_limit"]["unit"],
            )
        except Exception as e:
            raise e

    @property
    def buck_output_current_limit(self):
        """
        Compute for the target buck output current limit set
        using DAC data and scale and offset factors based on resistor divider network at ISET2P node.

        returns buck target output current limit and DAC output voltage in volts (V)
        return type: (float, float)

        """
        return (
            self.read_value(
                self.dac.voltage12,
                "volt",
                self._board_config["buck_output_current_limit"]["scale"],
                self._board_config["buck_output_current_limit"]["offset"],
            ),
            self.dac.voltage12.volt / 1000,
        )

    @buck_output_current_limit.setter
    def buck_output_current_limit(self, value):
        """Set buck output current limit V2 side. Valid values are 0 to 35."""
        try:
            self.set_value(
                value,
                self.dac.voltage12,
                "volt",
                self._board_config["buck_output_current_limit"]["scale"],
                self._board_config["buck_output_current_limit"]["offset"],
                self._board_config["buck_output_current_limit"]["min"],
                self._board_config["buck_output_current_limit"]["max"],
                self._board_config["buck_output_current_limit"]["unit"],
            )
        except Exception as e:
            raise e

    @property
    def boost_output_voltage(self):
        """Read boost output voltage at V1 terminals. Convert ADC data using scale and offset factors based on resistor divider network."""
        return self.read_value(
            self.adc.channel[14],
            "raw",
            self._board_config["boost_output_voltage"]["scale"] * self.adc_scale,
            self._board_config["boost_output_voltage"]["offset"],
        )

    @property
    def boost_target_output_voltage(self):
        """
        Compute for the target boost output voltage set at V1 side
        using DAC data and scale and offset factors based on resistor divider network at FB1 node.

        returns boost target output voltage and DAC output voltage in volts (V)
        return type: (float, float)

        """
        return (
            self.read_value(
                self.dac.voltage0,
                "volt",
                self._board_config["boost_target_output_voltage"]["scale"],
                self._board_config["boost_target_output_voltage"]["offset"],
            ),
            self.dac.voltage0.volt / 1000,
        )

    @boost_target_output_voltage.setter
    def boost_target_output_voltage(self, value):
        """Set boost output voltage at V1 side. Valid values are 14 to 56."""
        try:
            self.set_value(
                value,
                self.dac.voltage0,
                "volt",
                self._board_config["boost_target_output_voltage"]["scale"],
                self._board_config["boost_target_output_voltage"]["offset"],
                self._board_config["boost_target_output_voltage"]["min"],
                self._board_config["boost_target_output_voltage"]["max"],
                self._board_config["boost_target_output_voltage"]["unit"],
            )
        except Exception as e:
            raise e

    @property
    def boost_input_voltage(self):
        """Read boost input voltage at V2 terminals. Convert ADC data using scale and offset factors based on resistor divider network."""
        return self.read_value(
            self.adc.channel[4],
            "raw",
            self._board_config["boost_input_voltage"]["scale"] * self.adc_scale,
            self._board_config["boost_input_voltage"]["offset"],
        )

    @property
    def boost_output_current(self):
        """Read boost output current at V1 terminals. Convert ADC data using scale and offset factors based on resistor divider at IMON1 node."""
        return self.read_value(
            self.adc.channel[0],
            "raw",
            self._board_config["boost_output_current"]["scale"] * self.adc_scale,
            self._board_config["boost_output_current"]["offset"],
        )

    @property
    def boost_input_current(self):
        """Read boost input current at V2 terminals. Convert ADC data using scale and offset factors based on resistor divider at IMON2 node."""
        return self.read_value(
            self.adc.channel[8],
            "raw",
            self._board_config["boost_input_current"]["scale"] * self.adc_scale,
            self._board_config["boost_input_current"]["offset"],
        )

    @property
    def boost_input_undervoltage(self):
        """
        Compute for the boost input undervoltage set at V2 side
        using DAC data and scale and offset factors based on resistor divider network at UV2 node.

        returns boost input undervoltage and DAC output voltage in volts (V)
        return type: (float, float)

        """
        return (
            self.read_value(
                self.dac.voltage6,
                "volt",
                self._board_config["boost_input_undervoltage"]["scale"],
                self._board_config["boost_input_undervoltage"]["offset"],
            ),
            self.dac.voltage6.volt / 1000,
        )

    @boost_input_undervoltage.setter
    def boost_input_undervoltage(self, value):
        """Set boost input undervoltage V2 side. Valid values are 8 to 12."""
        try:
            self.set_value(
                value,
                self.dac.voltage6,
                "volt",
                self._board_config["boost_input_undervoltage"]["scale"],
                self._board_config["boost_input_undervoltage"]["offset"],
                self._board_config["boost_input_undervoltage"]["min"],
                self._board_config["boost_input_undervoltage"]["max"],
                self._board_config["boost_input_undervoltage"]["unit"],
            )
        except Exception as e:
            raise e

    @property
    def boost_input_current_limit(self):
        """
        Compute for the target boost input current limit set
        using DAC data and scale and offset factors based on resistor divider network at ISET2N node.

        returns boost target input current limit and DAC output voltage in volts (V)
        return type: (float, float)

        """
        return (
            self.read_value(
                self.dac.voltage14,
                "volt",
                self._board_config["boost_input_current_limit"]["scale"],
                self._board_config["boost_input_current_limit"]["offset"],
            ),
            self.dac.voltage14.volt / 1000,
        )

    @boost_input_current_limit.setter
    def boost_input_current_limit(self, value):
        """Set boost input current limit V2 side. Valid values are 0 to 35."""
        try:
            self.set_value(
                value,
                self.dac.voltage14,
                "volt",
                self._board_config["boost_input_current_limit"]["scale"],
                self._board_config["boost_input_current_limit"]["offset"],
                self._board_config["boost_input_current_limit"]["min"],
                self._board_config["boost_input_current_limit"]["max"],
                self._board_config["boost_input_current_limit"]["unit"],
            )
        except Exception as e:
            raise e

    @property
    def boost_output_current_limit(self):
        """
        Compute for the target boost output current limit set
        using DAC data and scale and offset factors based on resistor divider network at ISET1N node.

        returns boost target output current limit and DAC output voltage in volts (V)
        return type: (float, float)

        """
        return (
            self.read_value(
                self.dac.voltage10,
                "volt",
                self._board_config["boost_output_current_limit"]["scale"],
                self._board_config["boost_output_current_limit"]["offset"],
            ),
            self.dac.voltage10.volt / 1000,
        )

    @boost_output_current_limit.setter
    def boost_output_current_limit(self, value):
        """Set boost output current limit V1 side. Valid values are 0.07 to 10."""
        try:
            self.set_value(
                value,
                self.dac.voltage10,
                "volt",
                self._board_config["boost_output_current_limit"]["scale"],
                self._board_config["boost_output_current_limit"]["offset"],
                self._board_config["boost_output_current_limit"]["min"],
                self._board_config["boost_output_current_limit"]["max"],
                self._board_config["boost_output_current_limit"]["unit"],
            )
        except Exception as e:
            raise e

    def read_value(self, ctrl, ctrl_name, scale, offset):
        """Convert ADC data using the scale factor and offset based on simplified formulas derived and scaling
        factors derived using the resistor divider networks at the feedback nodes."""
        return (getattr(ctrl, ctrl_name) * scale) + offset

    def set_value(self, value, ctrl, ctrl_name, scale, offset, val_min, val_max, unit):
        """Convert user input value to equivalent DAC output voltage to control the output voltage and current limits
        using the scale factor and offset based on simplified formulas derived using the resistor
        divider networks at the feedback nodes."""
        if value < val_min or value > val_max:
            raise Exception(
                "Invalid value. Valid values for this property: {} to {} {}".format(
                    val_min, val_max, unit
                )
            )
        setattr(ctrl, ctrl_name, (value - offset) / scale)



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.context_manager import context_manager
from adi.rx_tx import tx
from adi.sync_start import sync_start


class ad9152(tx, context_manager, sync_start):
    """ AD9152 High-Speed DAC """

    _complex_data = False
    _tx_channel_names = ["voltage0", "voltage1"]
    _device_name = ""

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)

        self._txdac = self._ctx.find_device("axi-ad9152-hpc")

        tx.__init__(self)

    @property
    def sample_rate(self):
        """sample_rate: Sample rate RX and TX paths in samples per second"""
        return self._get_iio_attr("voltage0", "sampling_frequency", True, self._txdac)

    @sample_rate.setter
    def sample_rate(self, value):
        self._set_iio_attr("voltage0", "sampling_frequency", True, value, self._txdac)



# Copyright (C) 2023-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.ad7768 import ad7768_4


class cn0579(ad7768_4):

    """ CN0579 - Multichannel IEPE DAQ for CbM """

    def __init__(
        self, uri="ip:analog.local",
    ):

        ad7768_4.__init__(self, uri)

        self._gpio = self._ctx.find_device("cn0579_control")
        self._ad5696 = self._ctx.find_device("ad5696")

    @property
    def shift_voltage0(self):
        """shift_voltage: Shift voltage in mV from AD5696 to bias sensor data"""
        dac_chan = self._ad5696
        raw = self._get_iio_attr("voltage0", "raw", True, dac_chan)
        return raw  # * dac_scale * 1.22

    @shift_voltage0.setter
    def shift_voltage0(self, value):
        dac_chan = self._ad5696
        self._set_iio_attr_int("voltage0", "raw", True, int(value), dac_chan)

    @property
    def shift_voltage1(self):
        """shift_voltage: Shift voltage in mV from AD5696 to bias sensor data"""
        dac_chan = self._ad5696
        raw = self._get_iio_attr("voltage1", "raw", True, dac_chan)
        return raw  # * dac_scale * 1.22

    @shift_voltage1.setter
    def shift_voltage1(self, value):
        dac_chan = self._ad5696
        self._set_iio_attr_int("voltage1", "raw", True, int(value), dac_chan)

    @property
    def shift_voltage2(self):
        """shift_voltage: Shift voltage in mV from AD5696 to bias sensor data"""
        dac_chan = self._ad5696
        raw = self._get_iio_attr("voltage2", "raw", True, dac_chan)
        return raw  # * dac_scale * 1.22

    @shift_voltage2.setter
    def shift_voltage2(self, value):
        dac_chan = self._ad5696
        self._set_iio_attr_int("voltage2", "raw", True, int(value), dac_chan)

    @property
    def shift_voltage3(self):
        """shift_voltage: Shift voltage in mV from AD5696 to bias sensor data"""
        dac_chan = self._ad5696
        raw = self._get_iio_attr("voltage3", "raw", True, dac_chan)
        return raw  # * dac_scale * 1.22

    @shift_voltage3.setter
    def shift_voltage3(self, value):
        dac_chan = self._ad5696
        self._set_iio_attr_int("voltage3", "raw", True, int(value), dac_chan)

    @property
    def CC_CH0(self):
        """Get Channel 0 Current Source Control"""
        return self._get_iio_attr("voltage0", "raw", True, self._gpio)

    @CC_CH0.setter
    def CC_CH0(self, value):
        """Set Channel 0 Current Source Control"""
        self._set_iio_attr_int("voltage0", "raw", True, value, self._gpio)

    @property
    def CC_CH1(self):
        """Get Channel 1 Current Source Control"""
        return self._get_iio_attr("voltage1", "raw", True, self._gpio)

    @CC_CH1.setter
    def CC_CH1(self, value):
        """Set Channel 1 Current Source Control"""
        self._set_iio_attr_int("voltage1", "raw", True, value, self._gpio)

    @property
    def CC_CH2(self):
        """Get Channel 2 Current Source Control"""
        return self._get_iio_attr("voltage2", "raw", True, self._gpio)

    @CC_CH2.setter
    def CC_CH2(self, value):
        """Set Channel 2 Current Source Control"""
        self._set_iio_attr_int("voltage2", "raw", True, value, self._gpio)

    @property
    def CC_CH3(self):
        """Get Channel 3 Current Source Control"""
        return self._get_iio_attr("voltage3", "raw", True, self._gpio)

    @CC_CH3.setter
    def CC_CH3(self, value):
        """Set Channel 3 Current Source Control"""
        self._set_iio_attr_int("voltage3", "raw", True, value, self._gpio)



# Copyright (C) 2020-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class adf4371(attribute, context_manager):
    """ADF4371 Microwave Wideband Synthesizer
    with Integrated VCO

    parameters:
        uri: type=string
            URI of IIO context with ADF4371
    """

    _device_name = "adf4371"
    _muxout_options = (
        "tristate",
        "digital_lock",
        "charge_pump_up",
        "charge_pump_down",
        "RDIV2",
        "N_div_out",
        "VCO_test",
        "high",
        "VCO_calib_R_band",
        "VCO_calib_N_band",
    )

    def __init__(self, uri=""):
        context_manager.__init__(self, uri, self._device_name)

        # Find the device
        self._ctrl = self._ctx.find_device(self._device_name)

        # Raise an exception if the device isn't found
        if not self._ctrl:
            raise Exception("ADF4371 device not found")

    @property
    def muxout_mode(self):
        """Get/Set the MUX output mode"""
        return self._get_iio_dev_attr("muxout_mode", self._ctrl)

    @muxout_mode.setter
    def muxout_mode(self, value):
        """Get/Set the MUX output mode"""

        # Check that the value is valid
        if value.lower().strip() not in self._muxout_options:
            raise ValueError(
                f"muxout_mode of \"{value}\" is invalid. Valid options: {', '.join(self._muxout_options)}"
            )

        self._set_iio_dev_attr("muxout_mode", value, self._ctrl)

    @property
    def rf8_enable(self):
        """Get/Set the enable status of the 8GHz RF output"""
        return bool(
            1 - int(self._get_iio_attr("altvoltage0", "powerdown", True, self._ctrl))
        )

    @rf8_enable.setter
    def rf8_enable(self, value):
        """Get/Set the enable status of the 8GHz RF output"""
        self._set_iio_attr("altvoltage0", "powerdown", True, 1 - int(value), self._ctrl)

    @property
    def rf8_frequency(self):
        """Get/Set the frequency of the 8GHz RF output"""
        return self._get_iio_attr("altvoltage0", "frequency", True, self._ctrl)

    @rf8_frequency.setter
    def rf8_frequency(self, value):
        """Get/Set the frequency of the 8GHz RF output"""
        self._set_iio_attr("altvoltage0", "frequency", True, value, self._ctrl)

    @property
    def rfaux8_enable(self):
        """Get/Set the enable status of the Auxiliary 8GHz RF output"""
        return bool(
            1 - int(self._get_iio_attr("altvoltage1", "powerdown", True, self._ctrl))
        )

    @rfaux8_enable.setter
    def rfaux8_enable(self, value):
        """Get/Set the enable status of the Auxiliary 8GHz RF output"""
        self._set_iio_attr("altvoltage1", "powerdown", True, 1 - int(value), self._ctrl)

    @property
    def rfaux8_frequency(self):
        """Get/Set the frequency of the Auxiliary 8GHz RF output"""
        return self._get_iio_attr("altvoltage1", "frequency", True, self._ctrl)

    @rfaux8_frequency.setter
    def rfaux8_frequency(self, value):
        """Get/Set the frequency of the Auxiliary 8GHz RF output"""
        self._set_iio_attr("altvoltage1", "frequency", True, value, self._ctrl)

    @property
    def rf16_enable(self):
        """Get/Set the enable status of the 16GHz RF output"""
        return bool(
            1 - int(self._get_iio_attr("altvoltage2", "powerdown", True, self._ctrl))
        )

    @rf16_enable.setter
    def rf16_enable(self, value):
        """Get/Set the enable status of the 16GHz RF output"""
        self._set_iio_attr("altvoltage2", "powerdown", True, 1 - int(value), self._ctrl)

    @property
    def rf16_frequency(self):
        """Get/Set the frequency of the 16GHz RF output"""
        return self._get_iio_attr("altvoltage2", "frequency", True, self._ctrl)

    @rf16_frequency.setter
    def rf16_frequency(self, value):
        """Get/Set the frequency of the 16GHz RF output"""
        self._set_iio_attr("altvoltage2", "frequency", True, value, self._ctrl)

    @property
    def rf32_enable(self):
        """Get/Set the enable status of the 32GHz RF output"""
        return bool(
            1 - int(self._get_iio_attr("altvoltage3", "powerdown", True, self._ctrl))
        )

    @rf32_enable.setter
    def rf32_enable(self, value):
        """Get/Set the enable status of the 32GHz RF output"""
        self._set_iio_attr("altvoltage3", "powerdown", True, 1 - int(value), self._ctrl)

    @property
    def rf32_frequency(self):
        """Get/Set the frequency of the 32GHz RF output"""
        return self._get_iio_attr("altvoltage3", "frequency", True, self._ctrl)

    @rf32_frequency.setter
    def rf32_frequency(self, value):
        """Get/Set the frequency of the 32GHz RF output"""
        self._set_iio_attr("altvoltage3", "frequency", True, value, self._ctrl)

    @property
    def temperature(self):
        """Get the temperature reading"""
        return self._get_iio_attr("temp0", "input", False)



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

import numpy as np
from adi.context_manager import context_manager
from adi.rx_tx import rx


class ad4020(rx, context_manager):
    """AD4020 device"""

    _compatible_parts = [
        "ad4020",
        "ad4021",
        "ad4022",
    ]

    _device_name = ""
    _rx_data_type = np.int32
    _complex_data = False
    _rx_channel_names = ["voltage0"]

    def __init__(self, uri="", device_name="ad4020"):
        if not device_name:
            _device_name = self._compatible_parts[0]
        elif device_name not in self._compatible_parts:
            raise Exception(f"Not a compatible device: {device_name}")
        else:
            _device_name = device_name

        context_manager.__init__(self, uri, self._device_name)

        self._rxadc = self._ctx.find_device(_device_name)
        self._ctrl = self._ctx.find_device(_device_name)
        rx.__init__(self)

    @property
    def sampling_frequency(self):
        """Get and set the sampling frequency."""
        return self._get_iio_dev_attr("sampling_frequency")

    @sampling_frequency.setter
    def sampling_frequency(self, value):
        """Set the sampling frequency."""
        self._set_iio_dev_attr("sampling_frequency", str(value))


class ad4000(ad4020):
    _compatible_parts = [
        "ad4000",
        "ad4004",
        "ad4008",
    ]

    _rx_data_type = np.int16

    def __init__(self, uri="ip:analog.local", device_name="ad4000"):
        super().__init__(uri, device_name)


class ad4001(ad4020):
    _compatible_parts = [
        "ad4001",
        "ad4005",
    ]

    _rx_data_type = np.int16

    def __init__(self, uri="ip:analog.local", device_name="ad4001"):
        super().__init__(uri, device_name)


class ad4002(ad4020):
    _compatible_parts = [
        "ad4002",
        "ad4006",
        "ad4010",
    ]

    _rx_data_type = np.int32

    def __init__(self, uri="ip:analog.local", device_name="ad4002"):
        super().__init__(uri, device_name)


class ad4003(ad4020):
    _compatible_parts = [
        "ad4003",
        "ad4007",
        "ad4011",
    ]

    _rx_data_type = np.int32

    def __init__(self, uri="ip:analog.local", device_name="ad4003"):
        super().__init__(uri, device_name)



# Copyright (C) 2020-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class ltc2314_14(attribute, context_manager):
    """LTC2314-14 14-Bit, 4.5Msps Serial Sampling ADC

    parameters:
        uri: type=string
            URI of IIO context with LTC2314-14
    """

    _device_name = ""

    def __init__(self, uri=""):
        context_manager.__init__(self, uri, self._device_name)

        # Find the main and trigger devices
        self._ctrl = self._ctx.find_device("ltc2314-14")

        # Raise an exception if the device isn't found
        if not self._ctrl:
            raise Exception("LTC2314-14 device not found")

    @property
    def lsb_mv(self):
        """ Get the LSB in millivolts """
        return self._get_iio_attr("voltage0", "scale", False, self._ctrl)

    @property
    def voltage(self):
        """ Get the voltage reading from the ADC """
        code = self._get_iio_attr("voltage0", "raw", False, self._ctrl)
        return code * self.lsb_mv / 1000



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD
import numpy as np
from adi.ad9166 import ad9166 as ad9166_adi  # pyadi-iio library
from adi.context_manager import context_manager

try:
    import ad9166 as libad9166  # helper library for calibration function
except ImportError:
    libad9166 = None


class cn0511(ad9166_adi):
    """ CN0511 Raspberry Pi Hat Signal Generator """

    def __init__(self, uri=""):
        context_manager.__init__(self, uri, self._device_name)
        ad9166_adi.__init__(self, uri=uri)
        self._trim_dac_ch = "voltage0"
        self._trim_dac = self._ctx.find_device("ad5693r")
        self._amp = self._ctx.find_device("ad9166-amp")
        self.__calibrated_dev = self._ctx.find_device("ad9166")
        self.__ch = self.__calibrated_dev.find_channel("altvoltage0", True)
        if libad9166:
            self.__calibration_data = libad9166.find_calibration_data(
                self._ctx, "cn0511"
            )

        self.FIR85_enable = True
        self.sample_rate = 6000000000

    @property
    def trim_frequency_raw(self):
        """ trim_frequency_raw: modify output frequency of cn0511 in small steps"""
        return self._get_iio_attr(self._trim_dac_ch, "raw", True, self._trim_dac)

    @trim_frequency_raw.setter
    def trim_frequency_raw(self, value):
        self._set_iio_attr(self._trim_dac_ch, "raw", True, value, self._trim_dac)

    @property
    def amp_enable(self):
        """ amp_enable: Enable or Disable the CN0511 ad9166 amplifier """
        return True if (self._get_iio_dev_attr("en", _ctrl=self._amp) == 1) else False

    @amp_enable.setter
    def amp_enable(self, value=True):
        if value:
            val = 1
        else:
            val = 0
        self._set_iio_dev_attr_str("en", val, _ctrl=self._amp)

    @property
    def amplitude_cal(self):
        """amplitude_cal: CN0511 amplitude calibration

           Options:
               True: If you set this to true, the output is calibrated.
               False: Nothing happens.
        """
        return None

    @amplitude_cal.setter
    def amplitude_cal(self, value=True):
        if libad9166:
            if value:
                libad9166.device_set_iofs(
                    self.__calibrated_dev, self.__calibration_data, self.frequency
                )
        else:
            print("Warning: Missing libad9166, calibration failed.")

    @property
    def calibrated_output(self):
        """calibrated_output: ["desired_output_amplitude_in_dbm", "desired_output_frequency_in_Hz"]]"""
        if libad9166:
            return [int(20 * np.log10(self.raw / (2 ** 15))), self.frequency]
        print("Warning: Missing libad9166, calibration failed.")

    @calibrated_output.setter
    def calibrated_output(self, value):
        if libad9166:
            libad9166.set_amplitude(self.__calibrated_dev, value[0])
            libad9166.set_frequency(self.__ch, value[1])
            libad9166.device_set_iofs(
                self.__calibrated_dev, self.__calibration_data, value[1]
            )
        else:
            print("Warning: Missing libad9166, calibration failed.")

    @property
    def board_calibrated(self):
        """ board_calibrated: 1 if board was calibrated in production, 0 if board was not calibrated in production"""
        if libad9166:
            return libad9166.device_is_calibrated(self.__calibration_data)
        print("Warning: Missing libad9166, calibration failed.")



# Copyright (C) 2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.context_manager import context_manager
from adi.rx_tx import rx_def


class ad9213(rx_def, context_manager):
    """ AD9213 High-Speed ADC """

    _complex_data = False
    _rx_channel_names = ["voltage0"]
    _control_device_name = "axi-ad9213-rx-hpc"
    _rx_data_device_name = "axi-ad9213-rx-hpc"
    _device_name = ""



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.ad9152 import ad9152
from adi.ad9680 import ad9680


class DAQ3(ad9152, ad9680):
    """ DAQ3 High-Speed Data Aquistion Device """

    def __init__(self, uri=""):

        ad9152.__init__(self, uri=uri)
        ad9680.__init__(self, uri=uri)



# Copyright (C) 2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from typing import List

from adi.attribute import attribute
from adi.context_manager import context_manager


class tddn(context_manager, attribute):

    """TDDN Controller"""

    channel = []  # type: ignore
    _device_name: str = ""

    def __init__(self, uri=""):
        """TDDN Controller"""
        context_manager.__init__(self, uri, self._device_name)
        self._ctrl = self._ctx.find_device("iio-axi-tdd-0")

        for ch in self._ctrl.channels:
            name = ch._id
            self.channel.append(self._channel(self._ctrl, name))

    @property
    def frame_length_ms(self) -> float:
        """frame_length_ms: TDD frame length (ms)"""
        return self._get_iio_dev_attr("frame_length_ms")

    @frame_length_ms.setter
    def frame_length_ms(self, value: float):
        self._set_iio_dev_attr("frame_length_ms", value)

    @property
    def frame_length_raw(self) -> float:
        """frame_length_raw: TDD frame length (clock cycles)"""
        return self._get_iio_dev_attr("frame_length_raw")

    @frame_length_raw.setter
    def frame_length_raw(self, value: float):
        self._set_iio_dev_attr("frame_length_raw", value)

    @property
    def startup_delay_ms(self) -> float:
        """startup_delay_ms: Initial delay before the first frame (ms)"""
        return self._get_iio_dev_attr("startup_delay_ms")

    @startup_delay_ms.setter
    def startup_delay_ms(self, value: float):
        self._set_iio_dev_attr("startup_delay_ms", value)

    @property
    def startup_delay_raw(self) -> float:
        """startup_delay_raw: Initial delay before the first frame (clock cycles)"""
        return self._get_iio_dev_attr("startup_delay_raw")

    @startup_delay_raw.setter
    def startup_delay_raw(self, value: float):
        self._set_iio_dev_attr("startup_delay_raw", value)

    @property
    def internal_sync_period_ms(self) -> float:
        """internal_sync_period_ms: Period of the internal sync generator (ms)"""
        return self._get_iio_dev_attr("internal_sync_period_ms")

    @internal_sync_period_ms.setter
    def internal_sync_period_ms(self, value: float):
        self._set_iio_dev_attr("internal_sync_period_ms", value)

    @property
    def internal_sync_period_raw(self) -> float:
        """internal_sync_period_raw: Period of the internal sync generator (clock cycles)"""
        return self._get_iio_dev_attr("internal_sync_period_raw")

    @internal_sync_period_raw.setter
    def internal_sync_period_raw(self, value: float):
        self._set_iio_dev_attr("internal_sync_period_raw", value)

    @property
    def burst_count(self) -> int:
        """burst_count: Amount of frames to produce, where 0 means repeat indefinitely"""
        return self._get_iio_dev_attr("burst_count")

    @burst_count.setter
    def burst_count(self, value: int):
        self._set_iio_dev_attr("burst_count", value)

    @property
    def sync_soft(self) -> bool:
        """sync_soft: Trigger one software sync pulse"""
        return bool(int(self._get_iio_dev_attr("sync_soft")))

    @sync_soft.setter
    def sync_soft(self, value: bool):
        self._set_iio_dev_attr("sync_soft", int(value))

    @property
    def sync_external(self) -> bool:
        """sync_external: Enable the external sync trigger"""
        return bool(int(self._get_iio_dev_attr("sync_external")))

    @sync_external.setter
    def sync_external(self, value: bool):
        self._set_iio_dev_attr("sync_external", int(value))

    @property
    def sync_internal(self) -> bool:
        """sync_internal: Enable the internal sync trigger"""
        return bool(int(self._get_iio_dev_attr("sync_internal")))

    @sync_internal.setter
    def sync_internal(self, value: bool):
        self._set_iio_dev_attr("sync_internal", int(value))

    @property
    def sync_reset(self) -> bool:
        """sync_reset: Reset the internal counter when receiving a sync event"""
        return bool(int(self._get_iio_dev_attr("sync_reset")))

    @sync_reset.setter
    def sync_reset(self, value: bool):
        self._set_iio_dev_attr("sync_reset", int(value))

    @property
    def enable(self) -> bool:
        """enable: Enable or disable the TDD engine"""
        return bool(int(self._get_iio_dev_attr("enable")))

    @enable.setter
    def enable(self, value: bool):
        self._set_iio_dev_attr("enable", int(value))

    @property
    def state(self) -> int:
        """state: The current state of the internal FSM"""
        return self._get_iio_dev_attr("state")

    @state.setter
    def state(self, value: int):
        self._set_iio_dev_attr("state", value)

    class _channel(attribute):

        """TDDN channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def enable(self) -> bool:
            """channel_enable: Selected channel output enable"""
            return bool(int(self._get_iio_attr(self.name, "enable", True)))

        @enable.setter
        def enable(self, value: bool):
            self._set_iio_attr(self.name, "enable", True, int(value))

        @property
        def polarity(self) -> bool:
            """channel_polarity: Selected channel output polarity"""
            return bool(int(self._get_iio_attr(self.name, "polarity", True)))

        @polarity.setter
        def polarity(self, value: bool):
            self._set_iio_attr(self.name, "polarity", True, int(value))

        @property
        def on_ms(self) -> float:
            """channel_on_ms: The offset from the beginning of a new frame when the selected channel is set (ms)"""
            return self._get_iio_attr(self.name, "on_ms", True)

        @on_ms.setter
        def on_ms(self, value: float):
            self._set_iio_attr(self.name, "on_ms", True, value)

        @property
        def on_raw(self) -> float:
            """channel_on_raw: The offset from the beginning of a new frame when the selected channel is set (clock cycles)"""
            return self._get_iio_attr(self.name, "on_raw", True)

        @on_raw.setter
        def on_raw(self, value: float):
            self._set_iio_attr(self.name, "on_raw", True, value)

        @property
        def off_ms(self) -> float:
            """channel_off_ms: The offset from the beginning of a new frame when the selected channel is reset (ms)"""
            return self._get_iio_attr(self.name, "off_ms", True)

        @off_ms.setter
        def off_ms(self, value: float):
            self._set_iio_attr(self.name, "off_ms", True, value)

        @property
        def off_raw(self) -> float:
            """channel_off_raw: The offset from the beginning of a new frame when the selected channel is reset (clock cycles)"""
            return self._get_iio_attr(self.name, "off_raw", True)

        @off_raw.setter
        def off_raw(self, value: float):
            self._set_iio_attr(self.name, "off_raw", True, value)



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.ad9625 import ad9625
from adi.ada4961 import ada4961


class fmcadc3(ad9625, ada4961):

    """ FMCADC3 High-Speed Data Aquistion Device """

    def __init__(self, uri=""):

        ad9625.__init__(self, uri=uri)
        ada4961.__init__(self, uri=uri)



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class ad7291(context_manager):
    """ AD7291 ADC """

    _device_name = ""

    def __repr__(self):
        retstr = f"""
ad7291(uri="{self.uri}") object "{self._device_name}"
8-channel, I2C, 12-bit SAR ADC with temperature sensor

Channel layout:

voltageX.raw:              Raw 12-bit ADC code. read only for ADC channels
voltageX.scale:            ADC scale, millivolts per lsb
voltageX():                    Returns ADC reading in millivolts (read only)

temp0.raw:                      Temperature raw value
temp0.scale:                    Temperature scale value
temp0():                        Returns temperature in degrees Celsius

"""
        return retstr

    def __init__(self, uri="", device_index=0):

        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = ["ad7291"]

        self._ctrl = None
        index = 0

        # Selecting the device_index-th device from the 7291 family as working device.
        for device in self._ctx.devices:
            if device.name in compatible_parts:
                if index == device_index:
                    self._ctrl = device
                    break
                else:
                    index += 1

        for ch in self._ctrl.channels:
            name = ch._id
            if "temp" in name:

                setattr(self, name, self._temp_channel(self._ctrl, name))

            else:
                name = ch._id
                setattr(self, name, self._channel(self._ctrl, name))

    class _channel(attribute):
        """AD7291 channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """AD7291 channel raw value"""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def scale(self):
            """AD7291 channel scale(gain)"""
            return float(self._get_iio_attr_str(self.name, "scale", False))

        def __call__(self):
            """Utility function, returns millivolts"""
            return self.raw * self.scale

    class _temp_channel(_channel):  # attribute):
        """AD7291 temperature channel"""

        def __init__(self, ctrl, channel_name):
            super().__init__(ctrl, channel_name)

        @property
        def mean_raw(self):
            """AD7291 channel mean_raw value"""
            return self._get_iio_attr(self.name, "mean_raw", False)

        def __call__(self):
            """Utility function, returns deg. C"""
            return self.mean_raw * self.scale / 1000



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD
import numpy as np
from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class max31865(rx, context_manager, attribute):
    """MAX31865 RTD to Digital device"""

    _device_name = "max31865"
    _rx_data_type = np.int16
    _rx_unbuffered_data = True
    _rx_data_si_type = float

    def __init__(self, uri=""):
        context_manager.__init__(self, uri, self._device_name)
        self._ctrl = self._ctx.find_device("max31865")

        if self._ctrl is None:
            raise Exception("No device found")

        self.temp = self._channel_temp(self._ctrl, "temp")
        self.filter = self._channel_filt(self._ctrl, "filter")
        self._rx_channel_names = ["temp", "filter"]
        rx.__init__(self)

    @property
    def fault(self):
        """MAX31865 Over/Under Voltage Flag"""
        return self._get_iio_dev_attr("fault_ovuv")

    @property
    def samp_available(self):
        """MAX31865 Sampling frequency"""
        return self._get_iio_dev_attr("sampling_frequency_available")

    class _channel_temp(attribute):
        """MAX31865 temp channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """MAX31865 channel raw value"""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def scale(self):
            """MAX31865 channel scale value"""
            return self._get_iio_attr(self.name, "scale", False)

    class _channel_filt(attribute):
        """MAX31865 filter channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def notch(self):
            """MAX31865 Notch center frequency"""
            return self._get_iio_attr(self.name, "notch_center_frequency", False)

        @notch.setter
        def notch(self, value):
            """MAX31865 Notch center frequency (50 or 60)"""
            return self._set_iio_attr(self.name, "notch_center_frequency", False, value)



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class ada4961(context_manager, attribute):

    """ Low Distortion, 3.2 GHz, RF DGA """

    _device_name = ""

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)

        self._ctrl = self._ctx.find_device("ada4961")

        if not self._ctrl:
            raise Exception("ADF4159 device not found")

    @property
    def hardwaregain(self):
        """hardwaregain: Set hardware gain. Options are:
        up to 15 dB"""
        return self._get_iio_attr("voltage0", "hardwaregain", False, self._ctrl)

    @hardwaregain.setter
    def hardwaregain(self, value):
        self._set_iio_attr("voltage0", "hardwaregain", False, value, self._ctrl)



# Copyright (C) 2021-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class adl5960(attribute, context_manager):
    """ADL5960 10 MHz to 20 GHz, Integrated Vector Network Analyzer Front-End

    parameters:
        uri: type=string
            URI of IIO context with ADL5960
    """

    _device_name = "adl5960"

    def __init__(self, uri="", device_name=""):

        context_manager.__init__(self, uri, self._device_name)

        self._device_name = device_name
        # Find the device
        self._ctrl = self._ctx.find_device(self._device_name)

        # Raise an exception if the device isn't found
        if not self._ctrl:
            raise Exception("ADL5960 device not found")

    # Temp Sensor

    @property
    def temperature(self):
        """Get the temperature reading in °C"""
        return self._get_iio_attr("temp0", "input", False, self._ctrl) / 1000

    # LO input

    @property
    def lo_frequency(self):
        """Get/Set the frequency in Hz"""
        return self._get_iio_attr("altvoltage0", "frequency", False, self._ctrl)

    @property
    def lo_mode_available(self):
        """Get available LO modes"""
        return self._get_iio_attr_str(
            "altvoltage0", "mode_available", False, self._ctrl
        )

    @property
    def lo_mode(self):
        """Get/Set LO mode"""
        return self._get_iio_attr_str("altvoltage0", "mode", False, self._ctrl)

    @lo_mode.setter
    def lo_mode(self, value):
        """Get/Set LO mode"""
        self._set_iio_attr("altvoltage0", "mode", False, value, self._ctrl)

    # Offset input

    @property
    def offset_frequency(self):
        """Get/Set the offset frequency in Hz"""
        return self._get_iio_attr("altvoltage1", "offset_frequency", False, self._ctrl)

    @offset_frequency.setter
    def offset_frequency(self, value):
        """Get/Set the offset frequency in Hz"""
        self._set_iio_attr_int(
            "altvoltage1", "offset_frequency", False, int(value), self._ctrl
        )

    @property
    def if_frequency(self):
        """Get/Set the IF frequency in Hz"""
        return self._get_iio_attr("altvoltage1", "if_frequency", False, self._ctrl)

    @if_frequency.setter
    def if_frequency(self, value):
        """Get/Set the IF frequency in Hz"""
        self._set_iio_attr_int(
            "altvoltage1", "if_frequency", False, int(value), self._ctrl
        )

    @property
    def offset_mode_available(self):
        """Get available offset modes"""
        return self._get_iio_attr_str(
            "altvoltage1", "offset_mode_available", False, self._ctrl
        )

    @property
    def offset_mode(self):
        """Get/Set offset mode"""
        return self._get_iio_attr_str("altvoltage1", "offset_mode", False, self._ctrl)

    @offset_mode.setter
    def offset_mode(self, value):
        """Get/Set offset mode"""
        self._set_iio_attr("altvoltage1", "offset_mode", False, value, self._ctrl)

    # Forward

    @property
    def forward_gain(self):
        """Get/Set Forward Gain"""
        return self._get_iio_attr("voltage0", "forward_hardwaregain", True, self._ctrl)

    @forward_gain.setter
    def forward_gain(self, value):
        """Get/Set Forward Gain"""
        self._set_iio_attr_int(
            "voltage0", "forward_hardwaregain", True, int(value), self._ctrl
        )

    # Reflected

    @property
    def reflected_gain(self):
        """Get/Set Reflected Gain"""
        return self._get_iio_attr("voltage1", "reverse_hardwaregain", True, self._ctrl)

    @reflected_gain.setter
    def reflected_gain(self, value):
        """Get/Set Reflected Gain"""
        self._set_iio_attr_int(
            "voltage1", "reverse_hardwaregain", True, int(value), self._ctrl
        )

    # IF Filter cutoff

    @property
    def if_filter_cutoff(self):
        """Get/Set LPF 3db cutoff frequency (controls CIF1, CIF2)"""
        return self._get_iio_attr(
            "voltage0", "filter_low_pass_3db_frequency", True, self._ctrl
        )

    @if_filter_cutoff.setter
    def if_filter_cutoff(self, value):
        """Get/Set LPF 3db cutoff frequency (controls CIF1, CIF2)"""
        self._set_iio_attr_int(
            "voltage0", "filter_low_pass_3db_frequency", True, int(value), self._ctrl
        )

    def reg_read(self, reg):
        """Direct Register Access via debugfs"""
        self._set_iio_debug_attr_str("direct_reg_access", reg, self._ctrl)
        return self._get_iio_debug_attr_str("direct_reg_access", self._ctrl)

    def reg_write(self, reg, value):
        """Direct Register Access via debugfs"""
        self._set_iio_debug_attr_str("direct_reg_access", f"{reg} {value}", self._ctrl)



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

import iio


class context_manager(object):
    _uri_auto = "ip:analog"
    _ctx = None

    @property
    def ctx(self) -> iio.Context:
        """IIO Context"""
        return self._ctx

    def __init__(self, uri="", _device_name=""):
        if self._ctx:
            return
        self.uri = uri
        try:
            if self.uri == "":
                # Try USB contexts first
                if _device_name != "":
                    contexts = iio.scan_contexts()
                    for c in contexts:
                        if _device_name in contexts[c]:
                            self._ctx = iio.Context(c)
                            break
                # Try auto discover
                if not self._ctx and self._uri_auto != "":
                    self._ctx = iio.Context(self._uri_auto)
                if not self._ctx:
                    raise Exception("No device found")
            else:
                self._ctx = iio.Context(self.uri)
        except BaseException:
            raise Exception("No device found")



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.ad9162 import ad9162
from adi.ad9625 import ad9625


class FMComms11(
    ad9162, ad9625,
):
    """ FMCOMMS11 Transceiver """

    def __init__(self, uri=""):

        ad9162.__init__(self, uri=uri)
        ad9625.__init__(self, uri=uri)



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.context_manager import context_manager
from adi.jesd import jesd
from adi.obs import obs
from adi.rx_tx import rx_tx
from adi.sync_start import sync_start


class ad9371(rx_tx, context_manager, sync_start):
    """ AD9371 Transceiver """

    _complex_data = True
    _rx_channel_names = ["voltage0_i", "voltage0_q", "voltage1_i", "voltage1_q"]
    _tx_channel_names = ["voltage0", "voltage1", "voltage2", "voltage3"]
    _obs_channel_names = ["voltage0_i", "voltage0_q"]
    _device_name = ""

    def __init__(
        self, uri="", username="root", password="analog", disable_jesd_control=False
    ):
        """Initialize AD9371 interface class.
        Args:
            uri (str): URI of target platform with AD9371
            username (str): SSH username for target board. Required for JESD monitoring
            password (str): SSH password for target board. Required for JESD monitoring
            disable_jesd_control (bool): Disable JESD status monitoring over SSH
        """
        context_manager.__init__(self, uri, self._device_name)

        self._ctrl = self._ctx.find_device("ad9371-phy")
        self._rxadc = self._ctx.find_device("axi-ad9371-rx-hpc")
        self._rxobs = self._ctx.find_device("axi-ad9371-rx-obs-hpc")
        self._txdac = self._ctx.find_device("axi-ad9371-tx-hpc")
        self._ctx.set_timeout(30000)  # Needed for loading profiles

        if not disable_jesd_control and jesd:
            self._jesd = jesd(uri, username=username, password=password)

        rx_tx.__init__(self)

        self.obs = obs(self._ctx, self._rxobs, self._obs_channel_names)

    @property
    def ensm_mode(self):
        """ensm_mode: Enable State Machine State Allows real time control over
        the current state of the device. Options are: radio_on, radio_off"""
        return self._get_iio_dev_attr_str("ensm_mode")

    @ensm_mode.setter
    def ensm_mode(self, value):
        self._set_iio_dev_attr_str("ensm_mode", value)

    @property
    def gain_control_mode(self):
        """gain_control_mode: Mode of receive path AGC. Options are:
        automatic, hybrid, manual"""
        return self._get_iio_attr_str("voltage0", "gain_control_mode", False)

    @gain_control_mode.setter
    def gain_control_mode(self, value):
        self._set_iio_attr("voltage0", "gain_control_mode", False, value)

    @property
    def rx_quadrature_tracking_en_chan0(self):
        """Enable Quadrature tracking calibration for RX1"""
        return self._get_iio_attr("voltage0", "quadrature_tracking_en", False)

    @rx_quadrature_tracking_en_chan0.setter
    def rx_quadrature_tracking_en_chan0(self, value):
        self._set_iio_attr("voltage0", "quadrature_tracking_en", False, value)

    @property
    def rx_quadrature_tracking_en_chan1(self):
        """Enable Quadrature tracking calibration for RX2"""
        return self._get_iio_attr("voltage1", "quadrature_tracking_en", False)

    @rx_quadrature_tracking_en_chan1.setter
    def rx_quadrature_tracking_en_chan1(self, value):
        self._set_iio_attr("voltage1", "quadrature_tracking_en", False, value)

    @property
    def rx_hardwaregain_chan0(self):
        """rx_hardwaregain: Gain applied to RX path channel 0. Only applicable when
        gain_control_mode is set to 'manual'"""
        return self._get_iio_attr("voltage0", "hardwaregain", False)

    @rx_hardwaregain_chan0.setter
    def rx_hardwaregain_chan0(self, value):
        if self.gain_control_mode == "manual":
            self._set_iio_attr("voltage0", "hardwaregain", False, value)

    @property
    def rx_hardwaregain_chan1(self):
        """rx_hardwaregain: Gain applied to RX path channel 1. Only applicable when
        gain_control_mode is set to 'manual'"""
        return self._get_iio_attr("voltage1", "hardwaregain", False)

    @rx_hardwaregain_chan1.setter
    def rx_hardwaregain_chan1(self, value):
        if self.gain_control_mode == "manual":
            self._set_iio_attr("voltage1", "hardwaregain", False, value)

    @property
    def rx_temp_comp_gain_chan0(self):
        """rx_temp_comp_gain_chan0: """
        return self._get_iio_attr("voltage0", "temp_comp_gain", False)

    @rx_temp_comp_gain_chan0.setter
    def rx_temp_comp_gain_chan0(self, value):
        self._set_iio_attr("voltage0", "temp_comp_gain", False, value)

    @property
    def rx_temp_comp_gain_chan1(self):
        """rx_temp_comp_gain_chan1: """
        return self._get_iio_attr("voltage1", "temp_comp_gain", False)

    @rx_temp_comp_gain_chan1.setter
    def rx_temp_comp_gain_chan1(self, value):
        self._set_iio_attr("voltage1", "temp_comp_gain", False, value)

    @property
    def tx_quadrature_tracking_en_chan0(self):
        """Enable Quadrature tracking calibration for TX1"""
        return self._get_iio_attr("voltage0", "quadrature_tracking_en", True)

    @tx_quadrature_tracking_en_chan0.setter
    def tx_quadrature_tracking_en_chan0(self, value):
        self._set_iio_attr("voltage0", "quadrature_tracking_en", True, value)

    @property
    def tx_quadrature_tracking_en_chan1(self):
        """Enable Quadrature tracking calibration for TX2"""
        return self._get_iio_attr("voltage1", "quadrature_tracking_en", True)

    @tx_quadrature_tracking_en_chan1.setter
    def tx_quadrature_tracking_en_chan1(self, value):
        self._set_iio_attr("voltage1", "quadrature_tracking_en", True, value)

    @property
    def tx_hardwaregain_chan0(self):
        """tx_hardwaregain: Attenuation applied to TX path channel 0"""
        return self._get_iio_attr("voltage0", "hardwaregain", True)

    @tx_hardwaregain_chan0.setter
    def tx_hardwaregain_chan0(self, value):
        self._set_iio_attr("voltage0", "hardwaregain", True, value)

    @property
    def tx_hardwaregain_chan1(self):
        """tx_hardwaregain: Attenuation applied to TX path channel 1"""
        return self._get_iio_attr("voltage1", "hardwaregain", True)

    @tx_hardwaregain_chan1.setter
    def tx_hardwaregain_chan1(self, value):
        self._set_iio_attr("voltage1", "hardwaregain", True, value)

    @property
    def rx_rf_bandwidth(self):
        """rx_rf_bandwidth: Bandwidth of front-end analog filter of RX path"""
        return self._get_iio_attr("voltage0", "rf_bandwidth", False)

    @property
    def tx_rf_bandwidth(self):
        """tx_rf_bandwidth: Bandwidth of front-end analog filter of TX path"""
        return self._get_iio_attr("voltage0", "rf_bandwidth", True)

    @property
    def rx_enable_dec8(self):
        """rx_enable_dec8: Enable x8 decimation filter in RX path"""
        avail = self._get_iio_attr_str(
            "voltage0_i", "sampling_frequency_available", False, self._rxadc
        )
        avail = avail.strip().split(" ")
        val = self._get_iio_attr_str(
            "voltage0_i", "sampling_frequency", False, self._rxadc
        )
        return val == avail[1]

    @rx_enable_dec8.setter
    def rx_enable_dec8(self, value):
        avail = self._get_iio_attr_str(
            "voltage0_i", "sampling_frequency_available", False, self._rxadc
        )
        avail = sorted(avail.strip().split(" "))
        val = int(avail[1] if value else avail[0])
        self._set_iio_attr("voltage0_i", "sampling_frequency", False, val, self._rxadc)

    @property
    def tx_enable_int8(self):
        """tx_enable_int8: Enable x8 interpolation filter in TX path"""
        avail = self._get_iio_attr_str(
            "voltage0", "sampling_frequency_available", True, self._txdac
        )
        avail = avail.strip().split(" ")
        val = self._get_iio_attr_str(
            "voltage0", "sampling_frequency", True, self._txdac
        )
        return val == avail[1]

    @tx_enable_int8.setter
    def tx_enable_int8(self, value):
        avail = self._get_iio_attr_str(
            "voltage0", "sampling_frequency_available", True, self._txdac
        )
        avail = sorted(avail.strip().split(" "))
        val = int(avail[1] if value else avail[0])
        self._set_iio_attr("voltage0", "sampling_frequency", True, val, self._txdac)

    @property
    def rx_sample_rate(self):
        """rx_sample_rate: Sample rate RX path in samples per second
            This value will reflect the correct value when 8x decimator is enabled
        """
        dec = 8 if self.rx_enable_dec8 else 1
        return self._get_iio_attr("voltage0", "sampling_frequency", False) / dec

    @property
    def orx_sample_rate(self):
        """orx_sample_rate: Sample rate ORX path in samples per second
            This value will reflect the correct value when 8x decimator is enabled
        """
        dec = 8 if self.rx_enable_dec8 else 1
        return self._get_iio_attr("voltage2", "sampling_frequency", False) / dec

    @property
    def tx_sample_rate(self):
        """tx_sample_rate: Sample rate TX path in samples per second
            This value will reflect the correct value when 8x interpolator is enabled
        """
        dec = 8 if self.tx_enable_int8 else 1
        return self._get_iio_attr("voltage0", "sampling_frequency", True) / dec

    @property
    def rx_lo(self):
        """rx_lo: Carrier frequency of RX path"""
        return self._get_iio_attr("altvoltage0", "frequency", True)

    @rx_lo.setter
    def rx_lo(self, value):
        self._set_iio_attr("altvoltage0", "frequency", True, value)

    @property
    def tx_lo(self):
        """tx_lo: Carrier frequency of TX path"""
        return self._get_iio_attr("altvoltage1", "frequency", True)

    @tx_lo.setter
    def tx_lo(self, value):
        self._set_iio_attr("altvoltage1", "frequency", True, value)

    @property
    def sn_lo(self):
        """sn_lo: Carrier frequency of Sniffer/ORx path"""
        return self._get_iio_attr("altvoltage2", "frequency", True)

    @sn_lo.setter
    def sn_lo(self, value):
        self._set_iio_attr("altvoltage2", "frequency", True, value)

    @property
    def obs_gain_control_mode(self):
        """obs_gain_control_mode: Mode of Obs/Sniffer receive path AGC. Options are:
        automatic, hybrid, manual"""
        return self._get_iio_attr_str("voltage2", "gain_control_mode", False)

    @obs_gain_control_mode.setter
    def obs_gain_control_mode(self, value):
        self._set_iio_attr("voltage2", "gain_control_mode", False, value)

    @property
    def obs_hardwaregain(self):
        """obs_hardwaregain: Gain applied to Obs/Sniffer receive path chan0. Only applicable when
        obs_gain_control_mode is set to 'manual'"""
        return self._get_iio_attr("voltage2", "hardwaregain", False)

    @obs_hardwaregain.setter
    def obs_hardwaregain(self, value):
        if self.obs_gain_control_mode == "manual":
            self._set_iio_attr("voltage2", "hardwaregain", False, value)

    @property
    def obs_temp_comp_gain(self):
        """obs_temp_comp_gain: """
        return self._get_iio_attr("voltage2", "temp_comp_gain", False)

    @obs_temp_comp_gain.setter
    def obs_temp_comp_gain(self, value):
        self._set_iio_attr("voltage2", "temp_comp_gain", False, value)

    @property
    def obs_quadrature_tracking_en(self):
        """Enable Quadrature tracking calibration for OBS chan0"""
        return self._get_iio_attr("voltage2", "quadrature_tracking_en", False)

    @obs_quadrature_tracking_en.setter
    def obs_quadrature_tracking_en(self, value):
        self._set_iio_attr("voltage2", "quadrature_tracking_en", False, value)

    @property
    def obs_rf_port_select(self):
        """obs_rf_port_select: Observation path source. Options are:

        - OFF - SnRx path is disabled
        - ORX1_TX_LO – SnRx operates in observation mode on ORx1 with Tx LO synthesizer
        - ORX2_TX_LO – SnRx operates in observation mode on ORx2 with Tx LO synthesizer
        - INTERNALCALS – enables scheduled Tx calibrations while using SnRx path. The enableTrackingCals function needs to be called in RADIO_OFF state. It sets the calibration mask, which the scheduler will later use to schedule the desired calibrations. This command is issued in RADIO_OFF. Once the AD9371 moves to RADIO_ON state, the internal scheduler will use the enabled calibration mask to schedule calibrations whenever possible, based on the state of the transceiver. The Tx calibrations will not be scheduled until INTERNALCALS is selected and the Tx calibrations are enabled in the cal mask.
        - OBS_SNIFFER – SnRx operates in sniffer mode with latest selected Sniffer Input – for hardware pin control operation. In pin mode, the GPIO pins designated for ORX_MODE would select SNIFFER mode. Then MYKONOS_setSnifferChannel function would choose the channel.
        - ORX1_SN_LO – SnRx operates in observation mode on ORx1 with SNIFFER LO synthesizer
        - ORX2_SN_LO – SnRx operates in observation mode on ORx2 with SNIFFER LO synthesizer
        - SN_A – SnRx operates in sniffer mode on SnRxA with SNIFFER LO synthesizer
        - SN_B – SnRx operates in sniffer mode on SnRxB with SNIFFER LO synthesizer
        - SN_C – SnRx operates in sniffer mode on SnRxC with SNIFFER LO synthesizer

        """
        return self._get_iio_attr_str("voltage2", "rf_port_select", False)

    @obs_rf_port_select.setter
    def obs_rf_port_select(self, value):
        self._set_iio_attr("voltage2", "rf_port_select", False, value)

    @property
    def jesd204_statuses(self):
        return self._jesd.get_all_statuses()

    @property
    def profile(self):
        """Load profile file. Provide path to profile file to attribute"""
        return self._get_iio_dev_attr("profile_config")

    @profile.setter
    def profile(self, value):
        with open(value, "r") as file:
            data = file.read()
        self._set_iio_dev_attr_str("profile_config", data)


class ad9375(ad9371):
    """ AD9375 Transceiver """

    @property
    def tx_clgc_tracking_en_chan0(self):
        """Enable CLGC tracking for channel 0"""
        return self._get_iio_attr("voltage0", "clgc_tracking_en", True)

    @tx_clgc_tracking_en_chan0.setter
    def tx_clgc_tracking_en_chan0(self, value):
        self._set_iio_attr("voltage0", "clgc_tracking_en", True, value)

    @property
    def tx_clgc_tracking_en_chan1(self):
        """Enable CLGC tracking for channel 1"""
        return self._get_iio_attr("voltage1", "clgc_tracking_en", True)

    @tx_clgc_tracking_en_chan1.setter
    def tx_clgc_tracking_en_chan1(self, value):
        self._set_iio_attr("voltage1", "clgc_tracking_en", True, value)

    @property
    def tx_clgc_current_gain_chan0(self):
        if self.tx_clgc_tracking_en_chan0 == 1:
            """tx_clgc_current_gain: Current measured gain in 1/100ths dB scale in channel 0.
            Current GaindB = currentGain/100"""
            return self._get_iio_attr("voltage0", "clgc_current_gain", True)
        return

    @property
    def tx_clgc_current_gain_chan1(self):
        """tx_clgc_current_gain: Current measured gain in 1/100ths dB scale in channel 1.
        Current GaindB = currentGain/100"""
        if self.tx_clgc_tracking_en_chan1 == 1:
            return self._get_iio_attr("voltage1", "clgc_current_gain", True)
        return

    @property
    def tx_clgc_desired_gain_chan0(self):
        """tx_clgc_desired_gain: Desired gain from channel 0 output to orx input.
        Desired_gain (dB) = Desired_gain/100"""
        if self.tx_clgc_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "clgc_desired_gain", True) / 100
        return

    @tx_clgc_desired_gain_chan0.setter
    def tx_clgc_desired_gain_chan0(self, value):
        value *= 100
        self._set_iio_attr("voltage0", "clgc_desired_gain", True, value)

    @property
    def tx_clgc_desired_gain_chan1(self):
        """tx_clgc_desired_gain: Desired gain from channel 1 output to orx input.
        Desired_gain (dB) = Desired_gain/100"""
        if self.tx_clgc_tracking_en_chan1 == 1:
            return self._get_iio_attr("voltage1", "clgc_desired_gain", True) / 100
        return

    @tx_clgc_desired_gain_chan1.setter
    def tx_clgc_desired_gain_chan1(self, value):
        value *= 100
        self._set_iio_attr("voltage1", "clgc_desired_gain", True, value)

    @property
    def tx_clgc_orx_rms_chan0(self):
        """tx_clgc_orx_rms: RMS orx digital sample power measured in the DPD block on the orx side is returned
        with measurement resolution of 0.01 dB for channel 0. Prms dBFs = orxRMS/100"""
        if self.tx_clgc_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "clgc_orx_rms", True) / 100
        return

    @property
    def tx_clgc_track_count_chan0(self):
        """tx_clgc_track_count: The control reads back the number of times the CLGC has successfully run since
        CLGC initialization calibration for channel 0"""
        if self.tx_clgc_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "clgc_track_count", True)
        return

    @property
    def tx_clgc_track_count_chan1(self):
        """tx_clgc_track_count: The control reads back the number of times the CLGC has successfully run since
        CLGC initialization calibration for channel 1"""
        if self.tx_clgc_tracking_en_chan1 == 1:
            return self._get_iio_attr("voltage1", "clgc_track_count", True)
        return

    @property
    def tx_clgc_tx_gain_chan0(self):
        """tx_clgc_tx_gain: It controls the current channel 0 attenuation for a channel in 0.05 dB resolution.
        Tx_Attenuation(dB) = Tx_gain/200"""
        if self.tx_clgc_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "clgc_tx_gain", True) * 0.05
        return

    @property
    def tx_clgc_tx_gain_chan1(self):
        """tx_clgc_tx_gain: It controls the current channel 1 attenuation for a channel in 0.05 dB resolution. Tx_Attenuation(dB) = Tx_gain/200"""
        if self.tx_clgc_tracking_en_chan1 == 1:
            return self._get_iio_attr("voltage1", "clgc_tx_gain", True) * 0.05
        return

    @property
    def tx_clgc_tx_rms_chan0(self):
        """tx_clgc_tx_rms: The controls returns the RMS channel 0 digital sample power measured at DPD actuator output
        with measurement resolution of 0.01 dB. Prms dBFs = txRMS/100"""
        if self.tx_clgc_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "clgc_tx_rms", True) / 100
        return

    @property
    def tx_clgc_tx_rms_chan1(self):
        """tx_clgc_tx_rms: The controls returns the RMS channel 1 digital sample power measured at DPD actuator output
        with measurement resolution of 0.01 dB. Prms dBFs = txRMS/100"""
        if self.tx_clgc_tracking_en_chan1 == 1:
            return self._get_iio_attr("voltage1", "clgc_tx_rms", True) / 100
        return

    @property
    def tx_dpd_actuator_en_chan0(self):
        """Enable DPD actuator for channel 0"""
        return self._get_iio_attr("voltage0", "dpd_actuator_en", True)

    @tx_dpd_actuator_en_chan0.setter
    def tx_dpd_actuator_en_chan0(self, value):
        self._set_iio_attr("voltage0", "dpd_actuator_en", True, value)

    @property
    def tx_dpd_actuator_en_chan1(self):
        """Enable DPD actuator for channel 1"""
        return self._get_iio_attr("voltage1", "dpd_actuator_en", True)

    @tx_dpd_actuator_en_chan1.setter
    def tx_dpd_actuator_en_chan1(self, value):
        self._set_iio_attr("voltage1", "dpd_actuator_en", True, value)

    @property
    def tx_dpd_tracking_en_chan0(self):
        """Enable DPD tracking for channel 0"""
        return self._get_iio_attr("voltage0", "dpd_tracking_en", True)

    @tx_dpd_tracking_en_chan0.setter
    def tx_dpd_tracking_en_chan0(self, value):
        self._set_iio_attr("voltage0", "dpd_tracking_en", True, value)

    @property
    def tx_dpd_tracking_en_chan1(self):
        """Enable DPD tracking for channel 1"""
        return self._get_iio_attr("voltage1", "dpd_tracking_en", True)

    @tx_dpd_tracking_en_chan1.setter
    def tx_dpd_tracking_en_chan1(self, value):
        self._set_iio_attr("voltage1", "dpd_tracking_en", True, value)

    @property
    def tx_dpd_external_path_delay_chan0(self):
        """tx_dpd_external_path_delay: The control reads back the external path delay
        from channel 0 output to orx input at 1/16 sample resolution of the ORx sample rate"""
        if self.tx_dpd_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "dpd_external_path_delay", True) / 16
        return

    @property
    def tx_dpd_external_path_delay_chan1(self):
        """tx_dpd_external_path_delay: The control reads back the external path delay
        from channel 1 output to orx input at 1/16 sample resolution of the ORx sample rate"""
        if self.tx_dpd_tracking_en_chan1 == 1:
            return self._get_iio_attr("voltage1", "dpd_external_path_delay", True) / 16
        return

    @property
    def tx_dpd_model_error_chan0(self):
        """tx_dpd_model_error: The control reads back the percent error of the PA model ×10 to include 1 decimal place for channel 0"""
        if self.tx_dpd_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "dpd_model_error", True) / 10
        return

    @property
    def tx_dpd_model_error_chan1(self):
        """tx_dpd_model_error: The control reads back the percent error of the PA model ×10 to include 1 decimal place for channel 1"""
        if self.tx_dpd_tracking_en_chan1 == 1:
            return self._get_iio_attr("voltage1", "dpd_model_error", True) / 10
        return

    def tx_dpd_reset_en_chan0(self, value):
        """Enable DPD reset for channel 0"""
        if value == 1:
            self._set_iio_attr("voltage0", "dpd_reset_en", True, value)

    def tx_dpd_reset_en_chan1(self, value):
        """Enable DPD reset for channel 1"""
        if value == 1:
            self._set_iio_attr("voltage1", "dpd_reset_en", True, value)

    @property
    def tx_dpd_status_chan0(self):
        """tx_dpd_status: It reads back the DPD calibration status from the ARM processor for channel 0"""
        # DPD status lookup
        dpdstat_lookup = {
            0: "No Error",
            1: "Error: ORx disabled",
            2: "Error: Tx disabled",
            3: "Error: DPD initialization not run",
            4: "Error: Path delay not setup",
            5: "Error: ORx signal too low",
            6: "Error: ORx signal saturated",
            7: "Error: Tx signal too low",
            8: "Error: Tx signal saturated",
            9: "Error: Model error high",
            10: "Error: AM AM outliers",
            11: "Error: Invalid Tx profile",
            12: "Error: ORx QEC Disabled",
        }
        if self.tx_dpd_tracking_en_chan0 == 1:
            dpdstat_val = self._get_iio_attr("voltage0", "dpd_status", True)
            if dpdstat_val in dpdstat_lookup.keys():
                return dpdstat_lookup[dpdstat_val]
        return

    @property
    def tx_dpd_status_chan1(self):
        """tx_dpd_status: It reads back the DPD calibration status from the ARM processor for channel 1"""
        # DPD status lookup
        dpdstat_lookup = {
            0: "No Error",
            1: "Error: ORx disabled",
            2: "Error: Tx disabled",
            3: "Error: DPD initialization not run",
            4: "Error: Path delay not setup",
            5: "Error: ORx signal too low",
            6: "Error: ORx signal saturated",
            7: "Error: Tx signal too low",
            8: "Error: Tx signal saturated",
            9: "Error: Model error high",
            10: "Error: AM AM outliers",
            11: "Error: Invalid Tx profile",
            12: "Error: ORx QEC Disabled",
        }
        if self.tx_dpd_tracking_en_chan1 == 1:
            dpdstat_val = self._get_iio_attr("voltage1", "dpd_status", True)
            if dpdstat_val in dpdstat_lookup.keys():
                return dpdstat_lookup[dpdstat_val]
        return

    @property
    def tx_dpd_track_count_chan0(self):
        """tx_dpd_track_count: It reads back the number of times the DPD has successfully run since
        DPD initialization calibration for channel 0"""
        if self.tx_dpd_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "dpd_track_count", True)
        return

    @property
    def tx_dpd_track_count_chan1(self):
        """tx_dpd_track_count: It reads back the number of times the DPD has successfully run since
        DPD initialization calibration for channel 1"""
        if self.tx_dpd_tracking_en_chan1 == 1:
            return self._get_iio_attr("voltage1", "dpd_track_count", True)
        return

    @property
    def tx_vswr_tracking_en_chan0(self):
        """Enable VSWR tracking for channel 0"""
        return self._get_iio_attr("voltage0", "vswr_tracking_en", True)

    @tx_vswr_tracking_en_chan0.setter
    def tx_vswr_tracking_en_chan0(self, value):
        self._set_iio_attr("voltage0", "vswr_tracking_en", True, value)

    @property
    def tx_vswr_tracking_en_chan1(self):
        """Enable VSWR tracking for channel 1"""
        return self._get_iio_attr("voltage1", "vswr_tracking_en", True)

    @tx_vswr_tracking_en_chan1.setter
    def tx_vswr_tracking_en_chan1(self, value):
        self._set_iio_attr("voltage1", "vswr_tracking_en", True, value)

    @property
    def tx_vswr_forward_gain_chan0(self):
        """tx_vswr_forward: Forward rms gain measured from channel 0 to orx path. 0.01 dB = 1"""
        if self.tx_vswr_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "vswr_forward_gain", True) * 0.01
        return

    @property
    def tx_vswr_forward_gain_chan1(self):
        """tx_vswr_forward: Forward rms gain measured from channel 1 to orx path. 0.01 dB = 1"""
        if self.tx_vswr_tracking_en_chan1 == 1:
            return self._get_iio_attr("voltage1", "vswr_forward_gain", True) * 0.01
        return

    @property
    def tx_vswr_forward_gain_imag_chan0(self):
        """tx_vswr_forward_gain_imag: Imaginary part of the forward path complex gain for channel 0 (1 = 0.01 linear gain)"""
        if self.tx_vswr_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "vswr_forward_gain_imag", True) * 0.01
        return

    @property
    def tx_vswr_forward_gain_imag_chan1(self):
        if self.tx_vswr_tracking_en_chan1 == 1:
            """tx_vswr_forward_gain_imag: Imaginary part of the forward path complex gain for channel 1 (1 = 0.01 linear gain)"""
            return self._get_iio_attr("voltage1", "vswr_forward_gain_imag", True) * 0.01
        return

    @property
    def tx_vswr_forward_gain_real_chan0(self):
        """tx_vswr_forward_gain_real: Real part of the forward path complex gain for channel 0 (1 = 0.01 linear gain)"""
        if self.tx_vswr_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "vswr_forward_gain_real", True) * 0.01
        return

    @property
    def tx_vswr_forward_gain_real_chan1(self):
        """tx_vswr_forward_gain_real: Real part of the forward path complex gain for channel 1 (1 = 0.01 linear gain)"""
        if self.tx_vswr_tracking_en_chan1 == 1:
            return self._get_iio_attr("voltage1", "vswr_forward_gain_real", True) * 0.01
        return

    @property
    def tx_vswr_forward_orx_chan0(self):
        """tx_vswr_forward_orx: RMS Orx digital sample power measured at DPD block for ORx data in the forward measurement mode
        with measurement resolution of 0.01 dB and 21 dB offset for channel 0. Prms dBFS = txRms/100 + 21 dB"""
        if self.tx_vswr_tracking_en_chan0 == 1:
            return (self._get_iio_attr("voltage0", "vswr_forward_orx", True) / 100) + 21
        return

    @property
    def tx_vswr_forward_tx_chan0(self):
        """tx_vswr_forward_tx: RMS Tx digital sample power measured at DPD block for ORx data in the forward measurement mode
       with measurement resolution of 0.01 dB and 21 dB offset for channel 0. Prms dBFS = txRms/100 + 21 dB"""
        if self.tx_vswr_tracking_en_chan0 == 1:
            return (self._get_iio_attr("voltage0", "vswr_forward_tx", True) / 100) + 21
        return

    @property
    def tx_vswr_forward_tx_chan1(self):
        """tx_vswr_forward_tx: RMS Tx digital sample power measured at DPD block for ORx data in the forward measurement mode
       with measurement resolution of 0.01 dB and 21 dB offset for channel 1. Prms dBFS = txRms/100 + 21 dB"""
        if self.tx_vswr_tracking_en_chan1 == 1:
            return (self._get_iio_attr("voltage1", "vswr_forward_tx", True) / 100) + 21
        return

    @property
    def tx_vswr_reflected_gain_chan0(self):
        """tx_vswr_reflected_gain: Reflected path gain in RMS for channel 0. 1 = 0.01 dB gain"""
        if self.tx_vswr_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "vswr_reflected_gain", True) * 0.01
        return

    @property
    def tx_vswr_reflected_gain_chan1(self):
        """tx_vswr_reflected_gain: Reflected path gain in RMS for channel 1. 1 = 0.01 dB gain"""
        if self.tx_vswr_tracking_en_chan1 == 1:
            return self._get_iio_attr("voltage1", "vswr_reflected_gain", True) * 0.01
        return

    @property
    def tx_vswr_reflected_gain_imag_chan0(self):
        """tx_vswr_reflected_gain_imag: Imaginary part of the reflected path complex gain for channel 0. 1 = 0.01 linear gain"""
        if self.tx_vswr_tracking_en_chan0 == 1:
            return (
                self._get_iio_attr("voltage0", "vswr_reflected_gain_imag", True) * 0.01
            )
        return

    @property
    def tx_vswr_reflected_gain_imag_chan1(self):
        """tx_vswr_reflected_gain_imag: Imaginary part of the reflected path complex gain for channel 1. 1 = 0.01 linear gain"""
        if self.tx_vswr_tracking_en_chan1 == 1:
            return (
                self._get_iio_attr("voltage1", "vswr_reflected_gain_imag", True) * 0.01
            )
        return

    @property
    def tx_vswr_reflected_gain_real_chan0(self):
        """tx_vswr_reflected_gain_real: Real part of the reflected path complex gain for channel 0. 1 = 0.01 linear gain"""
        if self.tx_vswr_tracking_en_chan0 == 1:
            return (
                self._get_iio_attr("voltage0", "vswr_reflected_gain_real", True) * 0.01
            )
        return

    @property
    def tx_vswr_reflected_gain_real_chan1(self):
        """tx_vswr_reflected_gain_real: Real part of the reflected path complex gain for channel 1. 1 = 0.01 linear gain"""
        if self.tx_vswr_tracking_en_chan1 == 1:
            return (
                self._get_iio_attr("voltage1", "vswr_reflected_gain_real", True) * 0.01
            )
        return

    @property
    def tx_vswr_reflected_orx_chan0(self):
        """tx_vswr_reflected_orx: RMS ORx digital sample power measured at DPD block for the ORx data in the reverse measurement mode
       with measurement resolution of 0.01 dB and 21 dB offset for channel 0. Prms dBFS = orxRms/100 + 21 dB"""
        if self.tx_vswr_tracking_en_chan0 == 1:
            return (
                self._get_iio_attr("voltage0", "vswr_reflected_orx", True) / 100
            ) + 21
        return

    @property
    def tx_vswr_reflected_tx_chan0(self):
        """tx_vswr_reflected_tx: RMS Tx digital sample power measured at DPD actuator for the reverse measurement
       with measurement resolution of 0.01 dB and 21 dB offset for channel 0. Prms dBFS = txRms/100 + 21 dB"""
        if self.tx_vswr_tracking_en_chan0 == 1:
            return (
                self._get_iio_attr("voltage0", "vswr_reflected_tx", True) / 100
            ) + 21
        return

    @property
    def tx_vswr_reflected_tx_chan1(self):
        """tx_vswr_reflected_tx: RMS Tx digital sample power measured at DPD actuator for the reverse measurement
       with measurement resolution of 0.01 dB and 21 dB offset for channel 1. Prms dBFS = txRms/100 + 21 dB"""
        if self.tx_vswr_tracking_en_chan1 == 1:
            return (
                self._get_iio_attr("voltage1", "vswr_reflected_tx", True) / 100
            ) + 21
        return

    @property
    def tx_vswr_track_count_chan0(self):
        """tx_vswr_track_count: The control reads back the number of times the VSWR has successfully run
        since VSWR initialization calibration for channel 0"""
        if self.tx_vswr_tracking_en_chan0 == 1:
            return self._get_iio_attr("voltage0", "vswr_track_count", True)
        return

    @property
    def tx_vswr_track_count_chan1(self):
        """tx_vswr_track_count: The control reads back the number of times the VSWR has successfully run since
        VSWR initialization calibration for channel 1"""
        if self.tx_vswr_tracking_en_chan1 == 1:
            return self._get_iio_attr("voltage1", "vswr_track_count", True)
        return



# Copyright (C) 2020-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from decimal import Decimal

from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import tx


class ad5754r(tx, context_manager):
    """ AD5754R DAC """

    _complex_data = False
    channel = []
    _device_name = ""

    def __init__(self, uri="", device_name=""):
        """ Constructor for AD5754R driver class """

        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = ["ad5754r"]
        self._ctrl = None

        if not device_name:
            device_name = compatible_parts[0]
        else:
            if device_name not in compatible_parts:
                raise Exception(
                    f"Not a compatible device: {device_name}. Supported device names "
                    f"are: {','.join(compatible_parts)}"
                )

        # Select the device matching device_name as working device
        for device in self._ctx.devices:
            if device.name == device_name:
                self._ctrl = device
                self._txdac = device
                break

        if not self._ctrl:
            raise Exception("Error in selecting matching device")

        if not self._txdac:
            raise Exception("Error in selecting matching device")

        self.output_bits = []
        self.channel = []
        self._tx_channel_names = []
        for ch in self._ctrl.channels:
            name = ch.id
            self.output_bits.append(ch.data_format.bits)
            self._tx_channel_names.append(name)
            self.channel.append(self._channel(self._ctrl, name))

        tx.__init__(self)

    @property
    def int_ref_powerup(self):
        """Get internal reference powerup"""
        return self._get_iio_dev_attr_str("int_ref_powerup")

    @property
    def int_ref_powerup_available(self):
        """Get list of all internal reference powerup settings"""
        return self._get_iio_dev_attr_str("int_ref_powerup_available")

    @int_ref_powerup.setter
    def int_ref_powerup(self, value):
        """Set internal reference powerup"""
        if value in self.int_ref_powerup_available:
            self._set_iio_dev_attr_str("int_ref_powerup", value)
        else:
            raise ValueError(
                "Error: internal reference powerup not supported \nUse one of: "
                + str(self.int_ref_powerup_available)
            )

    @property
    def clear_setting(self):
        """Get clear code setting"""
        return self._get_iio_dev_attr_str("clear_setting")

    @property
    def clear_setting_available(self):
        """Get list of all clear code settings"""
        return self._get_iio_dev_attr_str("clear_setting_available")

    @clear_setting.setter
    def clear_setting(self, value):
        """Set clear setting"""
        if value in self.clear_setting_available:
            self._set_iio_dev_attr_str("clear_setting", value)
        else:
            raise ValueError(
                "Error: clear setting not supported \nUse one of: "
                + str(self.clear_setting_available)
            )

    @property
    def sdo_disable(self):
        """Get sdo disable"""
        return self._get_iio_dev_attr_str("sdo_disable")

    @property
    def sdo_disable_available(self):
        """Get list of all sdo enable/disable settings"""
        return self._get_iio_dev_attr_str("sdo_disable_available")

    @sdo_disable.setter
    def sdo_disable(self, value):
        """Set sdo enable/disable setting"""
        if value in self.sdo_disable_available:
            self._set_iio_dev_attr_str("sdo_disable", value)
        else:
            raise ValueError(
                "Error: sdo setting not supported \nUse one of: "
                + str(self.sdo_disable_available)
            )

    @property
    def sampling_frequency(self):
        """Get sampling frequency"""
        return self._get_iio_dev_attr_str("sampling_frequency")

    @sampling_frequency.setter
    def sampling_frequency(self, value):
        """Set sampling frequency"""
        self._set_iio_dev_attr_str("sampling_frequency", value)

    @property
    def clamp_enable(self):
        """Get clamp_en setting"""
        return self._get_iio_dev_attr_str("clamp_enable")

    @property
    def clamp_enable_available(self):
        """Get list of all clamp_en settings"""
        return self._get_iio_dev_attr_str("clamp_enable_available")

    @clamp_enable.setter
    def clamp_enable(self, value):
        """Set clamp_en"""
        if value in self.clamp_enable_available:
            self._set_iio_dev_attr_str("clamp_enable", value)
        else:
            raise ValueError(
                "Error: clamp_en setting not supported \nUse one of: "
                + str(self.clamp_enable_available)
            )

    @property
    def tsd_enable(self):
        """Get tsd_en setting"""
        return self._get_iio_dev_attr_str("tsd_enable")

    @property
    def tsd_enable_available(self):
        """Get list of all tsd_en settings"""
        return self._get_iio_dev_attr_str("tsd_enable_available")

    @tsd_enable.setter
    def tsd_enable(self, value):
        """Set tsd_en"""
        if value in self.tsd_enable_available:
            self._set_iio_dev_attr_str("tsd_enable", value)
        else:
            raise ValueError(
                "Error: tsd_en setting not supported \nUse one of: "
                + str(self.tsd_enable_available)
            )

    @property
    def oc_tsd(self):
        """Get oc_tsd status"""
        return self._get_iio_dev_attr_str("oc_tsd")

    @property
    def oc_tsd_available(self):
        """Get list of all possible oc_tsd status"""
        return self._get_iio_dev_attr_str("oc_tsd_available")

    @property
    def all_chns_clear(self):
        """Get current all_chns_clear setting"""
        return self._get_iio_dev_attr_str("all_chns_clear")

    @property
    def all_chns_clear_available(self):
        """Get list of all all_chns_clear settings"""
        return self._get_iio_dev_attr_str("all_chns_clear_available")

    @all_chns_clear.setter
    def all_chns_clear(self, value):
        """Clear all channels"""
        if value in self.all_chns_clear_available:
            self._set_iio_dev_attr_str("all_chns_clear", value)
        else:
            raise ValueError(
                "Error: all_chns_clear setting not supported \nUse one of: "
                + str(self.all_chns_clear_available)
            )

    @property
    def sw_ldac_trigger(self):
        """Get sw_ldac_trigger setting"""
        return self._get_iio_dev_attr_str("all_chns_clear")

    @property
    def sw_ldac_trigger_available(self):
        """Get list of all sw_ldac_trigger settings"""
        return self._get_iio_dev_attr_str("sw_ldac_trigger_available")

    @sw_ldac_trigger.setter
    def sw_ldac_trigger(self, value):
        """Trigger software LDAC"""
        if value in self.sw_ldac_trigger_available:
            self._set_iio_dev_attr_str("sw_ldac_trigger", value)
        else:
            raise ValueError(
                "Error: sw_ldac_trigger setting not supported \nUse one of: "
                + str(self.sw_ldac_trigger_available)
            )

    @property
    def hw_ldac_trigger(self):
        """Get hw_ldac_trigger setting"""
        return self._get_iio_dev_attr_str("hw_ldac_trigger")

    @property
    def hw_ldac_trigger_available(self):
        """Get list of all hw_ldac_trigger settings"""
        return self._get_iio_dev_attr_str("hw_ldac_trigger_available")

    @hw_ldac_trigger.setter
    def hw_ldac_trigger(self, value):
        """Trigger hardware LDAC"""
        if value in self.hw_ldac_trigger_available:
            self._set_iio_dev_attr_str("hw_ldac_trigger", value)
        else:
            raise ValueError(
                "Error: hw_ldac_trigger setting not supported \nUse one of: "
                + str(self.hw_ldac_trigger_available)
            )

    class _channel(attribute):
        """AD5754R channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """Get channel raw value
                DAC code in the range 0-65535"""
            return self._get_iio_attr(self.name, "raw", True)

        @raw.setter
        def raw(self, value):
            """Set channel raw value"""
            self._set_iio_attr(self.name, "raw", True, str(int(value)))

        @property
        def offset(self):
            """Get channel offset"""
            return self._get_iio_attr_str(self.name, "offset", True)

        @offset.setter
        def offset(self, value):
            """Set channel offset"""
            self._set_iio_attr(self.name, "offset", True, str(Decimal(value).real))

        @property
        def scale(self):
            """Get channel scale"""
            return float(self._get_iio_attr_str(self.name, "scale", True))

        @scale.setter
        def scale(self, value):
            """Set channel scale"""
            self._set_iio_attr(self.name, "scale", True, str(Decimal(value).real))

        @property
        def powerup(self):
            """Get DAC chn powerup"""
            return self._get_iio_attr_str(self.name, "powerup", True)

        @property
        def powerup_available(self):
            """Get list of DAC chn powerup settings"""
            return self._get_iio_attr_str(self.name, "powerup_available", True)

        @powerup.setter
        def powerup(self, value):
            """Set DAC chn powerup"""
            if value in self.powerup_available:
                self._set_iio_attr(self.name, "powerup", True, value)
            else:
                raise ValueError(
                    "Error: powerup setting not supported \nUse one of: "
                    + str(self.powerup_available)
                )

        @property
        def range(self):
            """Get output range"""
            return self._get_iio_attr_str(self.name, "range", True)

        @property
        def range_available(self):
            """Get list of all output ranges"""
            return self._get_iio_attr_str(self.name, "range_available", True)

        @range.setter
        def range(self, value):
            """Set DAC chn range"""
            if value in self.range_available:
                self._set_iio_attr(self.name, "range", True, value)
            else:
                raise ValueError(
                    "Error: range setting not supported \nUse one of: "
                    + str(self.range_available)
                )

        @property
        def dac_register(self):
            """Get dac_register value"""
            return self._get_iio_attr_str(self.name, "dac_register", True)

        @dac_register.setter
        def dac_register(self, value):
            """Set dac_register value"""
            self._set_iio_attr(
                self.name, "dac_register", True, str(Decimal(value).real)
            )



# Copyright (C) 2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from decimal import Decimal

import numpy as np
from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class ad405x(rx, context_manager):
    """ AD405x ADC """

    _complex_data = False
    channels = []  # type: ignore
    _device_name = ""

    def __init__(self, uri="", device_name=""):

        context_manager.__init__(self, uri, self._device_name)

        compatible_parts = ["ad4052", "ad4050"]

        self._ctrl = None

        if not device_name:
            device_name = compatible_parts[0]
        else:
            if device_name not in compatible_parts:
                raise Exception("Not a compatible device: " + device_name)

        # Select the device matching device_name as working device
        for device in self._ctx.devices:
            if device.name == device_name:
                self._ctrl = device
                self._rxadc = device
                break

        self.channels = []
        for ch in self._ctrl.channels:
            name = ch._id
            self._rx_channel_names.append(name)
            self.channels.append(self._channel(self._ctrl, name))

        rx.__init__(self)

    @property
    def operating_mode_avail(self):
        """Get available operating modes."""
        return self._get_iio_dev_attr_str("operating_mode_available")

    @property
    def operating_mode(self):
        """Get operating mode."""
        return self._get_iio_dev_attr_str("operating_mode")

    @operating_mode.setter
    def operating_mode(self, value):
        """Set operating mode."""
        if value in self.operating_mode_avail:
            self._set_iio_dev_attr_str("operating_mode", value)
        else:
            raise ValueError(
                "Error: Operating mode not supported \nUse one of: "
                + str(self.operating_mode_avail)
            )

    @property
    def burst_sample_rate(self):
        """Get burst sample rate. Only available in Burst Averaging Mode."""
        if "burst_sample_rate" in self._ctrl._attrs.keys():
            return self._get_iio_dev_attr("burst_sample_rate")
        else:
            raise ValueError(
                "Error: Burst sample rate not supported in " + self.operating_mode
            )

    @burst_sample_rate.setter
    def burst_sample_rate(self, value):
        """Set burst sample rate."""
        if "burst_sample_rate" in self._ctrl._attrs.keys():
            self._set_iio_dev_attr("burst_sample_rate", value)
        else:
            raise Exception(
                "Error: Burst sample rate not supported in " + self.operating_mode
            )

    @property
    def avg_filter_length_avail(self):
        """Get available average filter length. Only available in Burst Averaging Mode."""
        if "avg_filter_length_available" in self._ctrl._attrs.keys():
            return self._get_iio_dev_attr("avg_filter_length_available")
        else:
            raise Exception(
                "Error: Average filter length not supported in " + self.operating_mode
            )

    @property
    def avg_filter_length(self):
        """Get average filter length. Only available in Burst Averaging Mode."""
        if "avg_filter_length" in self._ctrl._attrs.keys():
            return self._get_iio_dev_attr("avg_filter_length")
        else:
            raise Exception(
                "Error: Average filter length not supported in " + self.operating_mode
            )

    @avg_filter_length.setter
    def avg_filter_length(self, value):
        """Set average filter length."""
        if "avg_filter_length_available" in self._ctrl._attrs.keys():
            if value in self.avg_filter_length_avail:
                self._set_iio_dev_attr("avg_filter_length", value)
            else:
                raise ValueError(
                    "Error: Average filter length not supported \nUse one of: "
                    + str(self.avg_filter_length_avail)
                )
        else:
            raise Exception(
                "Error: Average filter length not supported in " + self.operating_mode
            )

    @property
    def sampling_frequency(self):
        """Get sampling frequency."""
        return self._get_iio_dev_attr("sampling_frequency")

    @sampling_frequency.setter
    def sampling_frequency(self, value):
        """Set sampling_frequency."""
        self._set_iio_dev_attr("sampling_frequency", value)

    class _channel(attribute):
        """AD405x channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """AD405x channel raw value"""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def offset(self):
            """AD405x channel system calibration"""
            return self._get_iio_attr_str(self.name, "offset", False)

        @offset.setter
        def offset(self, value):
            self._set_iio_attr(self.name, "offset", False, str(Decimal(value).real))

        @property
        def scale(self):
            """AD405x channel scale"""
            return float(self._get_iio_attr_str(self.name, "scale", False))

        @scale.setter
        def scale(self, value):
            self._set_iio_attr(self.name, "scale", False, str(Decimal(value).real))

    def to_volts(self, index, val):
        """Converts raw value to SI"""
        _scale = self.channel[index].scale

        ret = None

        if isinstance(val, np.uint16):
            ret = val * _scale

        if isinstance(val, np.ndarray):
            ret = [x * _scale for x in val]

        return ret



# Copyright (C) 2021-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager


class genmux(attribute, context_manager):
    """GEN-MUX Generic IIO Mux device
    Control MUX devices via IIO device attributes

    parameters:
        uri: type=string
            URI of IIO context with GEN-MUX
    """

    _device_name = "gen-mux"

    def __init__(self, uri="", device_name=""):

        context_manager.__init__(self, uri, self._device_name)

        self._device_name = device_name
        # Find the device
        self._ctrl = self._ctx.find_device(self._device_name)

        # Raise an exception if the device isn't found
        if not self._ctrl:
            raise Exception("GEN-MUX device not found")

    @property
    def select_available(self):
        """Get available MUX options"""
        return self._get_iio_dev_attr_str("mux_select_available", self._ctrl)

    @property
    def select(self):
        """Get/Set the MUX select"""
        return self._get_iio_dev_attr_str("mux_select", self._ctrl)

    @select.setter
    def select(self, value):
        """Get/Set the MUX select"""
        self._set_iio_dev_attr_str("mux_select", value, self._ctrl)



# Copyright (C) 2022-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from numpy import min


class _dec_int_fpga_filter:
    """Decimator and interpolator fpga filter controls"""

    def _get_rates(self, dev, output):
        """Get the decimation and interpolation rates"""
        sfa = self._get_iio_attr_str(
            "voltage0", "sampling_frequency_available", output, dev
        )
        rates = sfa.strip().replace("[", "").replace("]", "").split(" ")
        return [int(rate) for rate in rates]

    @property
    def rx_dec8_filter_en(self) -> bool:
        """rx_dec8_filter_en: Enable decimate by 8 filter in FPGA"""
        rates = self._get_rates(self._rxadc, False)
        sf = self._get_iio_attr("voltage0", "sampling_frequency", False, self._rxadc)
        return sf == min(rates)

    @rx_dec8_filter_en.setter
    def rx_dec8_filter_en(self, value: bool):
        """rx_dec8_filter_en: Enable decimate by 8 filter in FPGA"""
        rates = self._get_rates(self._rxadc, False)
        sr = rates[1] if value else rates[0]
        self._set_iio_attr("voltage0", "sampling_frequency", False, sr, self._rxadc)

    @property
    def tx_int8_filter_en(self) -> bool:
        """tx_int8_filter_en: Enable interpolate by 8 filter in FPGA"""
        rates = self._get_rates(self._txdac, True)
        sf = self._get_iio_attr("voltage0", "sampling_frequency", True, self._txdac)
        return sf == min(rates)

    @tx_int8_filter_en.setter
    def tx_int8_filter_en(self, value: bool):
        """tx_int8_filter_en: Enable interpolate by 8 filter in FPGA"""
        rates = self._get_rates(self._txdac, True)
        sr = rates[1] if value else rates[0]
        self._set_iio_attr("voltage0", "sampling_frequency", True, sr, self._txdac)



# Copyright (C) 2021-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class adpd410x(rx, context_manager):
    """ adpd410x Multimodal Sensor Front End """

    _complex_data = False
    channel = []  # type: ignore
    _rx_channel_names = [
        "channel0",
        "channel1",
        "channel2",
        "channel3",
        "channel4",
        "channel5",
        "channel6",
        "channel7",
    ]
    _device_name = ""

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)

        self._rxadc = self._ctx.find_device("adpd410x")

        self.channel = []
        for name in self._rx_channel_names:
            self.channel.append(self._channel(self._rxadc, name))

    @property
    def sampling_frequency(self):
        """Get sampling frequency of the adpd410x"""
        return self._get_iio_dev_attr_str("sampling_frequency", self._rxadc)

    @sampling_frequency.setter
    def sampling_frequency(self, value):
        """Sets sampling frequency of the adpd410x"""
        self._set_iio_dev_attr_str("sampling_frequency", value, self._rxadc)

    @property
    def last_timeslot(self):
        """Get last timeslot of the adpd410x"""
        return self._get_iio_dev_attr_str("last_timeslot", self._rxadc)

    @last_timeslot.setter
    def last_timeslot(self, value):
        """Sets last timeslot of the adpd410x"""
        self._set_iio_dev_attr_str("last_timeslot", value, self._rxadc)

    @property
    def operation_mode(self):
        """Get operation mode of the adpd410x"""
        return self._get_iio_dev_attr_str("operation_mode", self._rxadc)

    @operation_mode.setter
    def operation_mode(self, value):
        """Sets operation mode of the adpd410x"""
        self._set_iio_dev_attr_str("operation_mode", value, self._rxadc)

    class _channel(attribute):
        """adpd410x channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def raw(self):
            """ adpd410x channel raw value"""
            return self._get_iio_attr(self.name, "raw", False, self._ctrl)



# Copyright (C) 2019-2024 Analog Devices, Inc.
#
# SPDX short identifier: ADIBSD

from decimal import Decimal

import numpy as np
from adi.attribute import attribute
from adi.context_manager import context_manager
from adi.rx_tx import rx


class adxl345(rx, context_manager, attribute):
    """ ADXL345 3-axis accelerometer """

    _device_name = "adxl345"
    _rx_data_type = np.int32
    _rx_unbuffered_data = True
    _rx_data_si_type = float

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)
        self._ctrl = self._ctx.find_device("adxl345")
        self.accel_x = self._channel(self._ctrl, "accel_x")
        self.accel_y = self._channel(self._ctrl, "accel_y")
        self.accel_z = self._channel(self._ctrl, "accel_z")
        self._rxadc = self._ctx.find_device("adxl345")
        self._rx_channel_names = ["accel_x", "accel_y", "accel_z"]
        rx.__init__(self)

    @property
    def sampling_frequency_available(self):
        """Provides all available sampling frequency settings for the ADXL345 channels"""
        return self._get_iio_dev_attr("sampling_frequency_available")

    @property
    def sampling_frequency(self):
        """ADXL345 sampling frequency"""
        # Only need to consider one channel, all others follow
        return float(self._get_iio_attr_str("accel_x", "sampling_frequency", False))

    @sampling_frequency.setter
    def sampling_frequency(self, value):
        self._set_iio_attr(
            "accel_x", "sampling_frequency", False, str(Decimal(value).real)
        )

    class _channel(attribute):
        """ADXL345 channel"""

        def __init__(self, ctrl, channel_name):
            self.name = channel_name
            self._ctrl = ctrl

        @property
        def calibbias(self):
            """ADXL345 channel offset"""
            return self._get_iio_attr(self.name, "calibbias", False)

        @calibbias.setter
        def calibbias(self, value):
            self._set_iio_attr(self.name, "calibbias", False, value)

        @property
        def raw(self):
            """ADXL345 channel raw value"""
            return self._get_iio_attr(self.name, "raw", False)

        @property
        def scale(self):
            """ADXL345 channel scale(gain)"""
            return float(self._get_iio_attr_str(self.name, "scale", False))
